import { Subject } from 'rxjs/Subject';
import { GlobalPostionStrategy } from './position/global-position-strategy';
export class OverlayRef {
    /**
     * @param {?} portalHost
     * @param {?} pane
     * @param {?} ngZone
     * @param {?} overlayConfig
     */
    constructor(portalHost, pane, ngZone, overlayConfig) {
        this.portalHost = portalHost;
        this.pane = pane;
        this.ngZone = ngZone;
        this.overlayConfig = overlayConfig;
        this.backdropElement = null;
        this._backdropClick = new Subject();
        this.attachments = new Subject();
        this.detachments = new Subject();
    }
    /**
     * @return {?}
     */
    get overlayElement() {
        return this.pane;
    }
    /**
     * @param {?} portal
     * @return {?}
     */
    attach(portal) {
        let /** @type {?} */ attachResult = this.portalHost.attach(portal);
        this.updatePostion();
        if (this.overlayConfig.shade) {
            this.attachBackdrop();
        }
        this.attachments.next();
        return attachResult;
    }
    /**
     * @return {?}
     */
    detach() {
        this.detachBackdrop();
        let /** @type {?} */ detachmentResult = this.portalHost.detach();
        this.detachments.next();
        return detachmentResult;
    }
    /**
     * @return {?}
     */
    dispose() {
        if (this.positionStrategy) {
            this.positionStrategy.dispose();
        }
        this.detachBackdrop();
        this.portalHost.dispose();
        this.attachments.complete();
        this._backdropClick.complete();
        this.detachments.next();
        this.detachments.complete();
    }
    /**
     * @return {?}
     */
    hasAttached() {
        return this.portalHost.hasAttached();
    }
    /**
     * @return {?}
     */
    backdropClick() {
        return this._backdropClick.asObservable();
    }
    /**
     * @return {?}
     */
    updatePostion() {
        // TODO: move to config
        if (!this.positionStrategy) {
            this.positionStrategy = new GlobalPostionStrategy();
        }
        this.positionStrategy.apply(this.pane);
    }
    /**
     * @return {?}
     */
    attachBackdrop() {
        this.backdropElement = document.createElement('div');
        this.backdropElement.classList.add('cui-overlay-backdrop');
        this.pane.parentElement.insertBefore(this.backdropElement, this.pane);
        this.backdropElement.addEventListener('click', () => {
            this._backdropClick.next(null);
        });
    }
    /**
     * @return {?}
     */
    detachBackdrop() {
        let /** @type {?} */ backdropToDetach = this.backdropElement;
        if (backdropToDetach) {
            let /** @type {?} */ finishDetach = () => {
                if (backdropToDetach && backdropToDetach.parentNode) {
                    backdropToDetach.parentNode.removeChild(backdropToDetach);
                }
                if (this.backdropElement == backdropToDetach) {
                    this.backdropElement = null;
                }
            };
            // backdropToDetach.classList.remove('cui-overlay-backdrop');
            // backdropToDetach.addEventListener('transitionend', finishDetach);
            finishDetach();
        }
    }
}
function OverlayRef_tsickle_Closure_declarations() {
    /** @type {?} */
    OverlayRef.prototype.backdropElement;
    /** @type {?} */
    OverlayRef.prototype._backdropClick;
    /** @type {?} */
    OverlayRef.prototype.attachments;
    /** @type {?} */
    OverlayRef.prototype.detachments;
    /** @type {?} */
    OverlayRef.prototype.positionStrategy;
    /** @type {?} */
    OverlayRef.prototype.portalHost;
    /** @type {?} */
    OverlayRef.prototype.pane;
    /** @type {?} */
    OverlayRef.prototype.ngZone;
    /** @type {?} */
    OverlayRef.prototype.overlayConfig;
}
