/**
 * @abstract
 */
export class Portal {
    constructor() {
    }
    /**
     * @param {?} host
     * @return {?}
     */
    attach(host) {
        if (host == null) {
            throw Error('PortalHost cannot be null');
        }
        if (host.hasAttached()) {
            throw Error('PortalHost has been attached');
        }
        this._attachedHost = host;
        return (host.attach(this));
    }
    /**
     * @return {?}
     */
    detach() {
        let /** @type {?} */ host = this._attachedHost;
        if (host == null) {
            throw Error('There is no portal has been attached');
        }
        this._attachedHost = null;
        return host.detach();
    }
    /**
     * @return {?}
     */
    get isAttached() {
        return this._attachedHost != null;
    }
    /**
     * @param {?} host
     * @return {?}
     */
    setAttachedHost(host) {
        this._attachedHost = host;
    }
}
function Portal_tsickle_Closure_declarations() {
    /** @type {?} */
    Portal.prototype._attachedHost;
}
export class ComponentPortal extends Portal {
    /**
     * @param {?} component
     * @param {?=} viewContainerRef
     * @param {?=} injector
     */
    constructor(component, viewContainerRef = null, injector = null) {
        super();
        this.component = component;
        this.viewContainerRef = viewContainerRef;
        this.injector = injector;
    }
}
function ComponentPortal_tsickle_Closure_declarations() {
    /** @type {?} */
    ComponentPortal.prototype.component;
    /** @type {?} */
    ComponentPortal.prototype.viewContainerRef;
    /** @type {?} */
    ComponentPortal.prototype.injector;
}
export class TemplatePortal extends Portal {
    /**
     * @param {?} templateRef
     * @param {?} viewContainerRef
     */
    constructor(templateRef, viewContainerRef) {
        super();
        this.locals = new Map();
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
    }
    /**
     * @return {?}
     */
    get origin() {
        return this.templateRef.elementRef;
    }
    /**
     * @param {?} host
     * @param {?=} locals
     * @return {?}
     */
    attach(host, locals) {
        this.locals = locals == null ? new Map() : locals;
        return super.attach(host);
    }
    /**
     * @return {?}
     */
    detach() {
        this.locals = new Map();
        return super.detach();
    }
}
function TemplatePortal_tsickle_Closure_declarations() {
    /** @type {?} */
    TemplatePortal.prototype.templateRef;
    /** @type {?} */
    TemplatePortal.prototype.viewContainerRef;
    /** @type {?} */
    TemplatePortal.prototype.locals;
}
/**
 * @abstract
 */
export class BasePortalHost {
    constructor() {
        this._isDisposed = false;
    }
    /**
     * @return {?}
     */
    hasAttached() {
        return !!this._attachedPortal;
    }
    /**
     * @param {?} portal
     * @return {?}
     */
    attach(portal) {
        if (!portal) {
            throw Error('Portal cannot be null');
        }
        if (this.hasAttached()) {
            throw Error('Portal already attached');
        }
        if (this._isDisposed) {
            throw Error('Portal Host already disposed');
        }
        if (portal instanceof ComponentPortal) {
            this._attachedPortal = portal;
            return this.attachComponentPortal(portal);
        }
        else if (portal instanceof TemplatePortal) {
            this._attachedPortal = portal;
            return this.attachTemplatePortal(portal);
        }
        throw Error('unkown Portal type:' + typeof portal);
    }
    /**
     * @abstract
     * @template T
     * @param {?} portal
     * @return {?}
     */
    attachComponentPortal(portal) { }
    /**
     * @abstract
     * @param {?} portal
     * @return {?}
     */
    attachTemplatePortal(portal) { }
    /**
     * @return {?}
     */
    detach() {
        if (this._attachedPortal) {
            this._attachedPortal.setAttachedHost(null);
            this._attachedPortal = null;
        }
        this._invokeDisposeFn();
    }
    /**
     * @return {?}
     */
    dispose() {
        if (this.hasAttached()) {
            this.detach();
        }
        this._invokeDisposeFn();
        this._isDisposed = true;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    setDisposeFn(fn) {
        this._disposeFn = fn;
    }
    /**
     * @return {?}
     */
    _invokeDisposeFn() {
        if (this._disposeFn) {
            this._disposeFn();
            this._disposeFn = null;
        }
    }
}
function BasePortalHost_tsickle_Closure_declarations() {
    /** @type {?} */
    BasePortalHost.prototype._attachedPortal;
    /** @type {?} */
    BasePortalHost.prototype._disposeFn;
    /** @type {?} */
    BasePortalHost.prototype._isDisposed;
}
