import { Http } from '@angular/http';
import { Component, Input, Output } from '@angular/core';
import { NzModalService } from 'ng-zorro-antd';
export class ModalImageCropperComponent {
    /**
     * @param {?} modalService
     * @param {?} http
     */
    constructor(modalService, http) {
        this.modalService = modalService;
        this.http = http;
        this.imageUrl = "";
        this.title = '图片裁剪';
        this.cropText = '裁剪';
        this.maskClosable = false;
        this.uploadParamName = 'file';
        // 回传结果中表示期望获取数据的属性名
        this.property = "relativePath";
        this.modalWidth = 740;
        this.modalHeight = 480;
        this.handleOk = (e) => {
            setTimeout(() => {
                /* destroy方法可以传入onOk或者onCancel。默认是onCancel */
                this.currentModal.destroy();
                this.currentModal = null;
                this.imageUrl = "";
            }, 500);
        };
        this.handleCancel = (e) => {
            setTimeout(() => {
                /* destroy方法可以传入onOk或者onCancel。默认是onCancel */
                this.currentModal.destroy();
                this.currentModal = null;
                this.imageUrl = "";
            }, 500);
        };
    }
    /**
     * @param {?} contentTpl
     * @param {?} footerTpl
     * @return {?}
     */
    showModalForTemplate(contentTpl, footerTpl) {
        this.currentModal = this.modalService.open({
            title: this.title,
            content: contentTpl,
            footer: footerTpl,
            width: this.modalWidth,
            wrapClassName: 'vertical-center-modal',
            /**
             * @return {?}
             */
            onOk() {
            },
            /**
             * @return {?}
             */
            onCancel() {
                console.log('Click cancel');
            },
            maskClosable: false
        });
        console.log(this.currentModal);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    crop(event) {
        if (this.uploadUrl && this.property) {
            this.cropper.getCroppedCanvas().toBlob((blob) => {
                let /** @type {?} */ formData = new FormData();
                formData.append(this.uploadParamName, blob, 'croped.png');
                this.http.post(this.uploadUrl, formData)
                    .subscribe((r) => {
                    console.log(r);
                    this.result = r.json()[this.property];
                    console.log(this.result);
                });
            });
        }
        else {
            let /** @type {?} */ url = this.cropper.getCroppedCanvas({
                maxWidth: 2048,
                maxHeight: 1152,
                fillColor: '#fff',
                imageSmoothingEnabled: false,
                imageSmoothingQuality: 'high',
            }).toDataURL();
            this.result = url;
        }
        setTimeout(() => {
            /* destroy方法可以传入onOk或者onCancel。默认是onCancel */
            this.currentModal.destroy();
            this.currentModal = null;
            this.imageUrl = "";
        }, 500);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    getCropper(event) {
        this.cropper = event;
    }
    /**
     * @param {?} files
     * @return {?}
     */
    read(files) {
        return new Promise((resolve, reject) => {
            if (!files || files.length === 0) {
                resolve();
                return;
            }
            const /** @type {?} */ file = files[0];
            if (/^image\/\w+$/.test(file.type)) {
                const /** @type {?} */ reader = new FileReader();
                reader.onerror = reject;
                reader.onabort = reject;
                reader.onload = (e) => {
                    // debugger;
                    console.log(e);
                    this.imageUrl = reader.result;
                };
                reader.readAsDataURL(file);
            }
            else {
                reject('Please choose an image file.');
            }
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    selectImage(event) {
        this.read(event.target.files).then(() => {
            event.target.value = '';
        }).catch((e) => {
            event.target.value = '';
        });
    }
}
ModalImageCropperComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-modal-cropper',
                templateUrl: './modal-image-cropper.component.html',
                styleUrls: ['./modal-image-cropper.component.scss']
            },] },
];
/**
 * @nocollapse
 */
ModalImageCropperComponent.ctorParameters = () => [
    { type: NzModalService, },
    { type: Http, },
];
ModalImageCropperComponent.propDecorators = {
    'options': [{ type: Input },],
    'title': [{ type: Input },],
    'cropText': [{ type: Input },],
    'maskClosable': [{ type: Input },],
    'uploadUrl': [{ type: Input },],
    'uploadParamName': [{ type: Input },],
    'property': [{ type: Input },],
    'modalWidth': [{ type: Input },],
    'modalHeight': [{ type: Input },],
    'result': [{ type: Output },],
};
function ModalImageCropperComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    ModalImageCropperComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ModalImageCropperComponent.ctorParameters;
    /** @type {?} */
    ModalImageCropperComponent.propDecorators;
    /** @type {?} */
    ModalImageCropperComponent.prototype.cropper;
    /** @type {?} */
    ModalImageCropperComponent.prototype.imageUrl;
    /** @type {?} */
    ModalImageCropperComponent.prototype.options;
    /** @type {?} */
    ModalImageCropperComponent.prototype.title;
    /** @type {?} */
    ModalImageCropperComponent.prototype.cropText;
    /** @type {?} */
    ModalImageCropperComponent.prototype.maskClosable;
    /** @type {?} */
    ModalImageCropperComponent.prototype.uploadUrl;
    /** @type {?} */
    ModalImageCropperComponent.prototype.uploadParamName;
    /** @type {?} */
    ModalImageCropperComponent.prototype.property;
    /** @type {?} */
    ModalImageCropperComponent.prototype.modalWidth;
    /** @type {?} */
    ModalImageCropperComponent.prototype.modalHeight;
    /** @type {?} */
    ModalImageCropperComponent.prototype.result;
    /** @type {?} */
    ModalImageCropperComponent.prototype.currentModal;
    /** @type {?} */
    ModalImageCropperComponent.prototype.handleOk;
    /** @type {?} */
    ModalImageCropperComponent.prototype.handleCancel;
    /** @type {?} */
    ModalImageCropperComponent.prototype.modalService;
    /** @type {?} */
    ModalImageCropperComponent.prototype.http;
}
