import { Component, Input, Output, EventEmitter, ContentChild, ContentChildren, Inject, Optional } from '@angular/core';
import { ColTplDirective } from './col-tpl.directive';
import { CUI_ROOT_CONFIG } from '../../consoleui-config';
export class DataTableComponent {
    /**
     * @param {?} cuiRootConfig
     */
    constructor(cuiRootConfig) {
        this.cuiRootConfig = cuiRootConfig;
        // input, Output searchForm
        this.reload = new EventEmitter();
        this.select = new EventEmitter();
        this.selectionChange = new EventEmitter();
        this.isSelectAll = false;
        this.keepSelection = false; // 是否在加载数据后或刷新数据后 没有找到 selection 的项时，保持 selection 的项
        this.rowId = 'id';
        this.sortFromFirst = true;
        this.colTpls = {};
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set showSerialNumber(value) {
        this._showSerialNumber = value;
    }
    /**
     * @return {?}
     */
    get showSerialNumber() {
        return this._showSerialNumber || (this.cuiRootConfig && this.cuiRootConfig.dataTable && this.cuiRootConfig.dataTable.showSerialNumber);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set serialNumberLabel(value) {
        this._serialNumberLabel = value;
    }
    /**
     * @return {?}
     */
    get serialNumberLabel() {
        return this._serialNumberLabel ||
            (this.cuiRootConfig && this.cuiRootConfig.dataTable && this.cuiRootConfig.dataTable.serialNumberLabel) || '序号';
    }
    /**
     * @return {?}
     */
    get isMultipleSelect() {
        return this.selectType && this.selectType == 'checkbox';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.columnsVisible = this.columns ? this.columns.filter(it => it.visible !== false) : [];
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        // console.log(this.rowActions);
        // console.log(this._colTpls);
        this._colTpls.forEach(it => {
            this.colTpls[it.cuiColTpl] = it.templateRef;
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        // console.log(changes);
        // throw new Error("Method not implemented.");
        let /** @type {?} */ chgSelection = changes['selection'];
        if (chgSelection && chgSelection.isFirstChange()) {
            this._refreshSel();
        }
        let /** @type {?} */ chgData = changes['data'];
        // if (chgData && chgData.isFirstChange()) {
        if (chgData) {
            if (this.keepSelection) {
                this._refreshSel();
            }
            else {
                setTimeout(() => {
                    this._refreshStatus();
                }, 300);
            }
            // console.log(this.selection);
            // this._refreshSel();
        }
        let /** @type {?} */ paginationChange = changes['pagination'];
        if (paginationChange && paginationChange.currentValue) {
            this._refreshSort(paginationChange.currentValue);
        }
    }
    /**
     * @param {?} pagination
     * @return {?}
     */
    _refreshSort(pagination) {
        const /** @type {?} */ pageSort = pagination.sort;
        let /** @type {?} */ sorts = [];
        if (Array.isArray(pageSort)) {
            sorts = pageSort.map(it => {
                return { sortKey: it.property, sort: it.direction };
            });
        }
        else if (pageSort && (typeof pageSort == 'string') && pageSort != "") {
            sorts = pageSort.split(/;|; /)
                .map(it => it.trim())
                .map(it => {
                const /** @type {?} */ sortVals = it.split(/,|, /);
                if (!!sortVals && sortVals.length == 1) {
                    return { sortKey: sortVals[0], sort: undefined };
                }
                else if (!!sortVals && sortVals.length == 2) {
                    return { sortKey: sortVals[0], sort: sortVals[1] };
                }
                return undefined;
            })
                .filter(it => it !== undefined);
        }
        this.columns.forEach(col => {
            if (col.sortKey) {
                const /** @type {?} */ sort = sorts.find(it => it.sortKey == col.sortKey);
                if (!!sort) {
                    col.sort = (!!sort.sort && sort.sort.toUpperCase() == 'DESC') ? 'DESC' : 'ASC';
                }
                else {
                    col.sort = undefined;
                }
            }
        });
        if (sorts && sorts.length > 0) {
            pagination.sort = sorts.map(it => `${it.sortKey},${it.sort}`).join(";");
        }
        else {
            pagination.sort = undefined;
        }
    }
    /**
     * @return {?}
     */
    _refreshSel() {
        if (this.data && this.data.length > 0) {
            this.data.forEach(row => {
                row.checked = this.rowChecked(row);
            });
            this._refreshStatus(false);
        }
    }
    /**
     * @return {?}
     */
    fireReload() {
        this.reload.emit(this.pagination);
    }
    /**
     * @return {?}
     */
    selectAll() {
        if (!this.isMultipleSelect) {
            return;
        }
        this.isSelectAll = !this.isSelectAll;
        if (!this.isSelectAll) {
            this.selection = [];
        }
        else {
            this.selection = [...this.data];
        }
        this.selectionChange.emit(this.selection);
        // 过时的
        this.select.emit(this.selection && this.selection.map(it => it['id']));
    }
    /**
     * @param {?} item
     * @param {?} checked
     * @return {?}
     */
    selectItem(item, checked) {
        if (this.isMultipleSelect) {
            let /** @type {?} */ selection = [];
            if (!this.selection) {
                this.selection = [];
            }
            this.selection.forEach((val) => {
                if (val != item) {
                    let /** @type {?} */ d = this.data && this.data.filter(it => it == item);
                    if (d && d.length > 0) {
                        selection.push(val);
                    }
                }
            });
            if (checked) {
                selection.push(item);
            }
            this.selection = selection;
        }
        else {
            this.selection = [item];
        }
        this.selectionChange.emit(this.selection);
        // 过时的
        this.select.emit(this.selection && this.selection.map(it => it['id']));
    }
    /**
     * @return {?}
     */
    toggleComplexSearch() {
        this.isComplexSearch = !this.isComplexSearch;
    }
    /**
     * @param {?} row
     * @return {?}
     */
    rowChecked(row) {
        if (this.selection) {
            return this.selection.findIndex(it => this.isRowEqual(it, row)) >= 0;
        }
        return false;
    }
    /**
     * @param {?=} emit
     * @return {?}
     */
    _refreshStatus(emit = true) {
        const /** @type {?} */ allChecked = this.hasData && this.data.every(value => value.checked === true);
        const /** @type {?} */ allUnChecked = !this.hasData || this.data.every(value => !value.checked);
        this._allChecked = allChecked;
        this._indeterminate = (!allChecked) && (!allUnChecked);
        this._anyCheckable = this.hasData && this.data.some(it => it.checkable !== false);
        // this._disabledButton = !this._dataSet.some(value => value.checked);
        // this._checkedNumber = this._dataSet.filter(value => value.checked).length;
        if (this.keepSelection && this.selection && this.data) {
            this.data.map(it => {
                if (it.checked && !this.rowChecked(it)) {
                    this.selection = [...this.selection, it];
                }
                if (!it.checked && this.rowChecked(it)) {
                    this.selection = this.selection && this.selection.filter(row => !this.isRowEqual(it, row));
                }
            });
        }
        else {
            this.selection = this.data && this.data.filter(value => value.checked);
        }
        if (emit) {
            this.selectionChange.emit(this.selection);
            // 过时的
            this.select.emit(this.selection && this.selection.map(it => it['id']));
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    checkAll(value) {
        if (value) {
            this.data.filter(it => it.checkable !== false).forEach(data => data.checked = true);
        }
        else {
            this.data.filter(it => it.checkable !== false).forEach(data => data.checked = false);
        }
        this._refreshStatus();
    }
    /**
     * @param {?} item1
     * @param {?} item2
     * @return {?}
     */
    isRowEqual(item1, item2) {
        if (!!item1 && !!item2) {
            if (item1 == item2) {
                return true;
            }
            if (item1.equal instanceof Function) {
                return item1.equal(item2);
            }
            return !!item1[this.rowId] && item1[this.rowId] == item2[this.rowId];
        }
        return false;
    }
    /**
     * @return {?}
     */
    get hasData() {
        return this.data && this.data.length > 0;
    }
    /**
     * @param {?} col
     * @return {?}
     */
    updateSort(col) {
        const { showSort, sortKey, sort, sortSwitchNull } = col;
        if (!showSort) {
            return;
        }
        let /** @type {?} */ sortStr = sortKey;
        if (sort === 'ASC') {
            col.sort = 'DESC';
            sortStr = sortStr + ',' + 'DESC';
        }
        else if (sort === 'DESC' && sortSwitchNull !== false) {
            col.sort = undefined;
            sortStr = "";
        }
        else {
            col.sort = 'ASC';
            sortStr = sortStr + ',' + 'ASC';
        }
        this.pagination.sort = sortStr;
        if (this.sortFromFirst !== false) {
            this.pagination.number = 0;
        }
        this.fireReload();
    }
}
DataTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-data-table',
                templateUrl: './data-table.component.html',
                styleUrls: ['./data-table.component.scss']
            },] },
];
/**
 * @nocollapse
 */
DataTableComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [CUI_ROOT_CONFIG,] }, { type: Optional },] },
];
DataTableComponent.propDecorators = {
    'columns': [{ type: Input },],
    'data': [{ type: Input },],
    'pagination': [{ type: Input },],
    'selectType': [{ type: Input },],
    'rowActionTitle': [{ type: Input },],
    'loading': [{ type: Input },],
    'isComplexSearch': [{ type: Input },],
    'dynamicColumns': [{ type: Input },],
    'reload': [{ type: Output },],
    'select': [{ type: Output },],
    'selectionChange': [{ type: Output },],
    'selection': [{ type: Input },],
    'keepSelection': [{ type: Input },],
    'rowId': [{ type: Input },],
    'sortFromFirst': [{ type: Input },],
    'rowActions': [{ type: ContentChild, args: ['rowActions',] },],
    '_colTpls': [{ type: ContentChildren, args: [ColTplDirective,] },],
    'complexSearch': [{ type: ContentChild, args: ['complexSearch',] },],
    'simpleSearch': [{ type: ContentChild, args: ['simpleSearch',] },],
    'listTools': [{ type: ContentChild, args: ['listTools',] },],
    'actionGroup': [{ type: ContentChild, args: ['actionGroup',] },],
    'showSerialNumber': [{ type: Input },],
    'serialNumberLabel': [{ type: Input },],
};
function DataTableComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    DataTableComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    DataTableComponent.ctorParameters;
    /** @type {?} */
    DataTableComponent.propDecorators;
    /** @type {?} */
    DataTableComponent.prototype.columns;
    /** @type {?} */
    DataTableComponent.prototype.data;
    /** @type {?} */
    DataTableComponent.prototype.pagination;
    /** @type {?} */
    DataTableComponent.prototype.selectType;
    /** @type {?} */
    DataTableComponent.prototype.rowActionTitle;
    /** @type {?} */
    DataTableComponent.prototype.loading;
    /** @type {?} */
    DataTableComponent.prototype.isComplexSearch;
    /** @type {?} */
    DataTableComponent.prototype.dynamicColumns;
    /** @type {?} */
    DataTableComponent.prototype.reload;
    /** @type {?} */
    DataTableComponent.prototype.select;
    /** @type {?} */
    DataTableComponent.prototype.selectionChange;
    /** @type {?} */
    DataTableComponent.prototype.isSelectAll;
    /** @type {?} */
    DataTableComponent.prototype.selection;
    /** @type {?} */
    DataTableComponent.prototype.keepSelection;
    /** @type {?} */
    DataTableComponent.prototype.rowId;
    /** @type {?} */
    DataTableComponent.prototype.sortFromFirst;
    /** @type {?} */
    DataTableComponent.prototype.rowActions;
    /** @type {?} */
    DataTableComponent.prototype._colTpls;
    /** @type {?} */
    DataTableComponent.prototype.complexSearch;
    /** @type {?} */
    DataTableComponent.prototype.simpleSearch;
    /** @type {?} */
    DataTableComponent.prototype.listTools;
    /** @type {?} */
    DataTableComponent.prototype.actionGroup;
    /** @type {?} */
    DataTableComponent.prototype.columnsVisible;
    /** @type {?} */
    DataTableComponent.prototype.colTpls;
    /** @type {?} */
    DataTableComponent.prototype._allChecked;
    /** @type {?} */
    DataTableComponent.prototype._indeterminate;
    /** @type {?} */
    DataTableComponent.prototype._anyCheckable;
    /** @type {?} */
    DataTableComponent.prototype._showSerialNumber;
    /** @type {?} */
    DataTableComponent.prototype._serialNumberLabel;
    /** @type {?} */
    DataTableComponent.prototype.cuiRootConfig;
}
