import { Component, Input, Output, EventEmitter, ContentChild } from '@angular/core';
export class PaginationComponent {
    constructor() {
        this.paginationChange = new EventEmitter();
        this.goto = new EventEmitter();
        this.totalPagesNumber = null;
        this.PER_PAGE_SIZES = [
            { value: 10, label: '10 / 页' },
            { value: 20, label: '20 / 页' },
            { value: 50, label: '50 / 页' },
            { value: 100, label: '100 / 页' },
            { value: 200, label: '200 / 页' },
            { value: 500, label: '500 / 页' },
            { value: 1000, label: '1000 / 页' },
        ];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // this.pagination = new PaginationModel(0, 0, 0);
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        let /** @type {?} */ pageChange = changes['pagination'];
        if (pageChange) {
            let /** @type {?} */ matched = this.PER_PAGE_SIZES.filter(it => {
                return it.value <= pageChange.currentValue.totalElements || it.value <= pageChange.currentValue.size;
            });
            this.PER_PAGE_SIZES = this.PER_PAGE_SIZES.slice(0, matched.length + 1);
            this.totalPagesNumber = pageChange.currentValue.number + 1;
        }
    }
    /**
     * @return {?}
     */
    prev() {
        if (!this.pagination.first
            && this.pagination.number >= 0) {
            this.pagination.number -= 1;
        }
        this.paginationChange.emit(this.pagination);
        this.gotoPage();
    }
    /**
     * @return {?}
     */
    next() {
        if (!this.pagination.last
            && this.pagination.number + 1 < this.pagination.totalPages) {
            this.pagination.number += 1;
        }
        this.paginationChange.emit(this.pagination);
        this.gotoPage();
    }
    /**
     * @return {?}
     */
    first() {
        if (!this.pagination.first) {
            this.pagination.number = 0;
        }
        this.paginationChange.emit(this.pagination);
        this.gotoPage();
    }
    /**
     * @return {?}
     */
    last() {
        if (!this.pagination.last) {
            this.pagination.number = this.pagination.totalPages - 1;
        }
        this.paginationChange.emit(this.pagination);
        this.gotoPage();
    }
    /**
     * @return {?}
     */
    gotoPage() {
        this.goto.emit(this.pagination.number);
    }
    /**
     * @param {?} $event
     * @param {?} page
     * @return {?}
     */
    gotoTotalPages($event, page) {
        if ($event.keyCode) {
            if ($event.keyCode == 13) {
                this.validation(page);
            }
        }
        else {
            this.validation(page);
        }
    }
    /**
     * @param {?} page
     * @return {?}
     */
    validation(page) {
        const /** @type {?} */ reg = new RegExp("^[0-9]*$");
        if (!reg.test(page.toString())) {
            this.totalPagesNumber = this.pagination.number + 1;
        }
        else {
            if (page > 0 && page <= this.pagination.totalPages) {
                this.pagination.number = page - 1;
                this.paginationChange.emit(this.pagination);
                this.gotoPage();
            }
            else {
                this.totalPagesNumber = this.pagination.number + 1;
            }
        }
    }
}
PaginationComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-pagination',
                templateUrl: './pagination.component.html',
                styleUrls: ['./pagination.component.scss']
            },] },
];
/**
 * @nocollapse
 */
PaginationComponent.ctorParameters = () => [];
PaginationComponent.propDecorators = {
    'pagination': [{ type: Input },],
    'paginationChange': [{ type: Output, args: ['paginationChange',] },],
    'goto': [{ type: Output, args: ['goto',] },],
    'paginationTemplate': [{ type: ContentChild, args: ['paginationTemplate',] },],
};
function PaginationComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    PaginationComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    PaginationComponent.ctorParameters;
    /** @type {?} */
    PaginationComponent.propDecorators;
    /** @type {?} */
    PaginationComponent.prototype.pagination;
    /** @type {?} */
    PaginationComponent.prototype.paginationChange;
    /** @type {?} */
    PaginationComponent.prototype.goto;
    /** @type {?} */
    PaginationComponent.prototype.totalPagesNumber;
    /** @type {?} */
    PaginationComponent.prototype.PER_PAGE_SIZES;
    /** @type {?} */
    PaginationComponent.prototype.paginationTemplate;
}
