import { Injectable } from '@angular/core';
import { defaultTreeConfig } from '../defs';
import { TreeNode } from './tree-node';
import * as _ from 'lodash';
const { pick } = _;
export class TreeModel {
    constructor() {
        // _roots: CuiTreeNode[];
        this.config = defaultTreeConfig;
        this.selection = [];
    }
    /**
     * @return {?}
     */
    get roots() {
        return this.virtualRoot.children;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    fireEvent(event) {
        let /** @type {?} */ e = this.events[event.name];
        if (e) {
            e.emit(event.data);
        }
    }
    /**
     * @param {?} node
     * @param {?=} event
     * @return {?}
     */
    addSelection(node, event) {
        if (!this.isSelected(node)) {
            this.selection = [...this.selection || [], node];
            this.fireEvent({ name: 'nodeSelect', data: { originalEvent: event, node: node } });
        }
        this.fireEvent({ name: 'selectionChange', data: this.selection });
    }
    /**
     * @param {?} node
     * @param {?=} event
     * @return {?}
     */
    removeSelection(node, event) {
        if (this.isSelected(node)) {
            this.selection = this.selection.filter((v, i) => v != node);
            this.fireEvent({ name: 'nodeUnselect', data: { originalEvent: event, node: node } });
        }
        this.fireEvent({ name: 'selectionChange', data: this.selection });
    }
    /**
     * @param {?} node
     * @return {?}
     */
    isSelected(node) {
        return this.findIndexInSelection(node) != -1;
    }
    /**
     * @param {?} __0
     * @return {?}
     */
    setData({ nodes, config = null, events }) {
        if (config) {
            this.config = Object.assign({}, defaultTreeConfig, config);
        }
        if (nodes) {
            this.nodes = nodes;
        }
        if (events) {
            this.events = events;
        }
        this.update();
    }
    /**
     * @return {?}
     */
    update() {
        if (this.selection) {
            this.selection = [];
        }
        let /** @type {?} */ virtualRootConfig = {
            virtual: true,
            [this.config.data.key.children]: this.nodes,
            hasChildren: this.config.async && this.config.async.enable || this.nodes
        };
        this.virtualRoot = new TreeNode(virtualRootConfig, null, this, 0);
        if (this.config.async && this.config.async.enable && !this.nodes) {
            this.virtualRoot.loadChildren();
        }
        // this.roots = this.virtualRoot.children;
    }
    /**
     * @param {?} node
     * @return {?}
     */
    findIndexInSelection(node) {
        let /** @type {?} */ index = -1;
        if (this.selection) {
            index = this.selection.findIndex((n, i) => n == node || n.id == node.id);
        }
        return index;
    }
    /**
     * @param {?} filterFn
     * @param {?=} startNodes
     * @return {?}
     */
    filter(filterFn, startNodes) {
        let /** @type {?} */ startWith = [];
        if (startNodes) {
            if (Array.isArray(startNodes)) {
                startWith = startNodes;
            }
            else {
                startWith = [startNodes];
            }
        }
        startWith = startWith.filter(it => !it.data['virtual']);
        startWith = startWith && startWith.length > 0 ? startWith : this.roots;
        return this._filter(filterFn, startWith);
    }
    /**
     * @param {?} filterFn
     * @param {?} startWith
     * @return {?}
     */
    _filter(filterFn, startWith) {
        let /** @type {?} */ result = [];
        if (!startWith || startWith.length == 0) {
            return result;
        }
        result = startWith.filter(it => filterFn(it));
        startWith.forEach(it => {
            if (it.hasChildren && it.children && it.children.length > 0) {
                let /** @type {?} */ sr = this._filter(filterFn, it.children);
                result = result ? [...result, ...sr] : sr;
            }
        });
        return result;
    }
}
TreeModel.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
TreeModel.ctorParameters = () => [];
function TreeModel_tsickle_Closure_declarations() {
    /** @type {?} */
    TreeModel.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    TreeModel.ctorParameters;
    /** @type {?} */
    TreeModel.prototype.nodes;
    /** @type {?} */
    TreeModel.prototype.config;
    /** @type {?} */
    TreeModel.prototype.selection;
    /** @type {?} */
    TreeModel.prototype.virtualRoot;
    /** @type {?} */
    TreeModel.prototype.events;
}
