import { Component, ElementRef, Input, Output, EventEmitter, ContentChild, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomHandler } from '../../core/dom/dom-handler';
export const /** @type {?} */ CHIPS_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => ChipsComponent),
    multi: true
};
export class ChipsComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     */
    constructor(el, domHandler) {
        this.el = el;
        this.domHandler = domHandler;
        this.allowDuplicate = true;
        this.onAdd = new EventEmitter();
        this.onRemove = new EventEmitter();
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        /* this.templates.forEach((item) => {
          switch (item.getType()) {
            case 'item':
              this.itemTemplate = item.template;
              break;
    
            default:
              this.itemTemplate = item.template;
              break;
          }
        }); */
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.value = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setDisabledState(val) {
        this.disabled = val;
    }
    /**
     * @param {?} data
     * @param {?} field
     * @return {?}
     */
    resolveFieldData(data, field) {
        if (data && field) {
            if (field.indexOf('.') == -1) {
                return data[field];
            }
            else {
                let /** @type {?} */ fields = field.split('.');
                let /** @type {?} */ value = data;
                for (let /** @type {?} */ i = 0, /** @type {?} */ len = fields.length; i < len; ++i) {
                    value = value[fields[i]];
                }
                return value;
            }
        }
        else {
            return null;
        }
    }
    /**
     * @return {?}
     */
    onFocus() {
        this.focus = true;
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.focus = false;
        this.onModelTouched();
    }
    /**
     * @param {?} event
     * @param {?} index
     * @return {?}
     */
    removeItem(event, index) {
        if (this.disabled) {
            return;
        }
        let /** @type {?} */ removedItem = this.value[index];
        this.value = this.value.filter((val, i) => i != index);
        this.onModelChange(this.value);
        this.onRemove.emit({
            originalEvent: event,
            value: removedItem
        });
    }
    /**
     * @param {?} event
     * @param {?} inputEL
     * @return {?}
     */
    onKeydown(event, inputEL) {
        switch (event.which) {
            // backspace
            case 8:
                if (inputEL.value.length === 0 && this.value && this.value.length > 0) {
                    this.value = [...this.value];
                    let /** @type {?} */ removedItem = this.value.pop();
                    this.onModelChange(this.value);
                    this.onRemove.emit({
                        originalEvent: event,
                        value: removedItem
                    });
                }
                break;
            // enter
            case 13:
                this.value = this.value || [];
                if (inputEL.value && inputEL.value.trim().length && (!this.max || this.max > this.value.length)) {
                    if (this.allowDuplicate || !this.value.includes(inputEL.value)) {
                        this.value = [...this.value, inputEL.value];
                        this.onModelChange(this.value);
                        this.onAdd.emit({
                            originalEvent: event,
                            value: inputEL.value
                        });
                    }
                }
                inputEL.value = '';
                event.preventDefault();
                break;
            default:
                if (this.max && this.value && this.max === this.value.length) {
                    event.preventDefault();
                }
                break;
        }
    }
    /**
     * @return {?}
     */
    get maxedOut() {
        return this.max && this.value && this.max === this.value.length;
    }
}
ChipsComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-chips',
                templateUrl: './chips.component.html',
                styleUrls: ['./chips.component.scss'],
                providers: [CHIPS_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
ChipsComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
];
ChipsComponent.propDecorators = {
    'style': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'disabled': [{ type: Input },],
    'field': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'max': [{ type: Input },],
    'tabindex': [{ type: Input },],
    'inputId': [{ type: Input },],
    'allowDuplicate': [{ type: Input },],
    'inputStyle': [{ type: Input },],
    'inputStyleClass': [{ type: Input },],
    'onAdd': [{ type: Output },],
    'onRemove': [{ type: Output },],
    'itemTemplate': [{ type: ContentChild, args: ['itemTemplate',] },],
};
function ChipsComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    ChipsComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ChipsComponent.ctorParameters;
    /** @type {?} */
    ChipsComponent.propDecorators;
    /** @type {?} */
    ChipsComponent.prototype.style;
    /** @type {?} */
    ChipsComponent.prototype.styleClass;
    /** @type {?} */
    ChipsComponent.prototype.disabled;
    /** @type {?} */
    ChipsComponent.prototype.field;
    /** @type {?} */
    ChipsComponent.prototype.placeholder;
    /** @type {?} */
    ChipsComponent.prototype.max;
    /** @type {?} */
    ChipsComponent.prototype.tabindex;
    /** @type {?} */
    ChipsComponent.prototype.inputId;
    /** @type {?} */
    ChipsComponent.prototype.allowDuplicate;
    /** @type {?} */
    ChipsComponent.prototype.inputStyle;
    /** @type {?} */
    ChipsComponent.prototype.inputStyleClass;
    /** @type {?} */
    ChipsComponent.prototype.onAdd;
    /** @type {?} */
    ChipsComponent.prototype.onRemove;
    /** @type {?} */
    ChipsComponent.prototype.itemTemplate;
    /** @type {?} */
    ChipsComponent.prototype.value;
    /** @type {?} */
    ChipsComponent.prototype.valueChanged;
    /** @type {?} */
    ChipsComponent.prototype.focus;
    /** @type {?} */
    ChipsComponent.prototype.onModelChange;
    /** @type {?} */
    ChipsComponent.prototype.onModelTouched;
    /** @type {?} */
    ChipsComponent.prototype.el;
    /** @type {?} */
    ChipsComponent.prototype.domHandler;
}
