import { Directive, EventEmitter, Output, Input } from '@angular/core';
/**
 * CKGroup component
 * Usage :
 *  <ckeditor [(ngModel)]="data" [config]="{...}" debounce="500">
 *      <ckbutton [name]="'SaveButton'" [command]="'saveCommand'" (click)="save($event)"
 *                [icon]="'/save.png'" [toolbar]="'customGroup,1'" [label]="'Save'">
 *      </ckbutton>
 *   </ckeditor>
 */
export class CKButtonDirective {
    constructor() {
        this.click = new EventEmitter();
    }
    /**
     * @param {?} editor
     * @return {?}
     */
    initialize(editor) {
        editor.instance.addCommand(this.command, {
            exec: (evt) => {
                this.click.emit(evt);
            },
        });
        editor.instance.ui.addButton(this.name, {
            label: this.label,
            command: this.command,
            toolbar: this.toolbar,
            icon: this.icon,
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.name)
            throw new Error('Attribute "name" is required on <ckbutton>');
        if (!this.command)
            throw new Error('Attribute "command" is required on <ckbutton>');
    }
}
CKButtonDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ckbutton',
            },] },
];
/**
 * @nocollapse
 */
CKButtonDirective.ctorParameters = () => [];
CKButtonDirective.propDecorators = {
    'click': [{ type: Output },],
    'label': [{ type: Input },],
    'command': [{ type: Input },],
    'toolbar': [{ type: Input },],
    'name': [{ type: Input },],
    'icon': [{ type: Input },],
};
function CKButtonDirective_tsickle_Closure_declarations() {
    /** @type {?} */
    CKButtonDirective.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    CKButtonDirective.ctorParameters;
    /** @type {?} */
    CKButtonDirective.propDecorators;
    /** @type {?} */
    CKButtonDirective.prototype.click;
    /** @type {?} */
    CKButtonDirective.prototype.label;
    /** @type {?} */
    CKButtonDirective.prototype.command;
    /** @type {?} */
    CKButtonDirective.prototype.toolbar;
    /** @type {?} */
    CKButtonDirective.prototype.name;
    /** @type {?} */
    CKButtonDirective.prototype.icon;
}
