// Imports
import { Component, Input, Output, ViewChild, EventEmitter, NgZone, forwardRef, ContentChildren, } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { CKButtonDirective } from './ckbutton.directive';
import { CKGroupDirective } from './ckgroup.directive';
/**
 * CKEditor component
 * Usage :
 *  <ckeditor [(ngModel)]="data" [config]="{...}" debounce="500"></ckeditor>
 */
export class CKEditorComponent {
    /**
     * Constructor
     * @param {?} zone
     */
    constructor(zone) {
        this.zone = zone;
        this.toolbarMode = "basic"; // for ueditor alias
        this.urlWithBasePath = true;
        this.change = new EventEmitter();
        this.editorChange = new EventEmitter();
        this.ready = new EventEmitter();
        this.blur = new EventEmitter();
        this.focus = new EventEmitter();
        this.contentDom = new EventEmitter();
        this.fileUploadRequest = new EventEmitter();
        this._value = '';
        this.defaultConfig = {
            toolbar_full: [
                ['Source', '-', 'Save', 'NewPage', 'Preview', '-', 'Templates'],
                ['Cut', 'Copy', 'Paste', 'PasteText', 'PasteFromWord', '-', 'Print', 'SpellChecker', 'Scayt'],
                ['Undo', 'Redo', '-', 'Find', 'Replace', '-', 'SelectAll', 'RemoveFormat'],
                ['Form', 'Checkbox', 'Radio', 'TextField', 'Textarea', 'Select', 'Button', 'ImageButton', 'HiddenField'],
                '/',
                ['Bold', 'Italic', 'Underline', 'Strike', '-', 'Subscript', 'Superscript'],
                ['NumberedList', 'BulletedList', '-', 'Outdent', 'Indent', 'Blockquote'],
                ['JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock'],
                ['Link', 'Unlink', 'Anchor'],
                ['Image', 'Flash', 'Table', 'HorizontalRule', 'Smiley', 'SpecialChar', 'PageBreak'],
                '/',
                ['Styles', 'Format', 'Font', 'FontSize'],
                ['TextColor', 'BGColor']
            ],
            toolbar_basic: [
                ['NewPage', 'Preview', '-', 'Undo', 'Redo'],
                ['Format', '-', 'Bold', 'Italic', 'Underline', 'Strike', '-', 'Subscript', 'Superscript'],
                ['NumberedList', 'BulletedList', '-', 'Outdent', 'Indent', 'Blockquote'],
                ['JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock'],
                ['Link', 'Unlink'],
                ['TextColor', 'BGColor'],
                ['Image']
            ],
            toolbar_simple: [
                ['Undo', 'Redo'],
                ['Format', '-', 'Bold', 'Italic', 'Underline', 'Strike'],
                ['JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock'],
                ['Link', 'Unlink'],
                ['TextColor', 'BGColor'],
                ['Image']
            ],
            toolbar: 'basic',
            removePlugins: 'elementspath',
            // filebrowserBrowseUrl: '/api/upload/file',
            filebrowserImageUploadUrl: '/api/ckeditor/upload/image?responseType=json',
            linkShowAdvancedTab: false,
            linkShowTargetTab: false,
        };
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set config(val) {
        this._config = Object.assign(this.defaultConfig, val);
    }
    /**
     * @return {?}
     */
    get config() {
        if (this.urlWithBasePath) {
            if (this._config) {
                const /** @type {?} */ separator = this._config.filebrowserImageUploadUrl && this._config.filebrowserImageUploadUrl.includes("?")
                    ? '&' : "?";
                this._config.filebrowserImageUploadUrl = this._config.filebrowserImageUploadUrl ?
                    this._config.filebrowserImageUploadUrl + `${separator}withBasePath=1` : null;
            }
            else {
                const /** @type {?} */ separator = this.defaultConfig.filebrowserImageUploadUrl && this.defaultConfig.filebrowserImageUploadUrl.includes("?")
                    ? '&' : "?";
                this.defaultConfig.filebrowserImageUploadUrl = this.defaultConfig.filebrowserImageUploadUrl ?
                    this.defaultConfig.filebrowserImageUploadUrl + `${separator}withBasePath=1` : null;
            }
        }
        return this._config || this.defaultConfig;
    }
    /**
     * @return {?}
     */
    get value() {
        return this._value;
    }
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = v;
            this.onChange(v);
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.readonly && this.instance) {
            this.instance.setReadOnly(changes.readonly.currentValue);
        }
    }
    /**
     * On component destroy
     * @return {?}
     */
    ngOnDestroy() {
        if (this.instance) {
            setTimeout(() => {
                this.instance.removeAllListeners();
                CKEDITOR.instances[this.instance.name].destroy();
                this.instance.destroy();
                this.instance = null;
            });
        }
    }
    /**
     * On component view init
     * @return {?}
     */
    ngAfterViewInit() {
        this.ckeditorInit(this.config || {});
    }
    /**
     * On component view checked
     * @return {?}
     */
    ngAfterViewChecked() {
        this.ckeditorInit(this.config || {});
    }
    /**
     * Value update process
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.zone.run(() => {
            this.value = value;
            this.onChange(value);
            this.onTouched();
            this.change.emit(value);
        });
    }
    /**
     * CKEditor init
     * @param {?} config
     * @return {?}
     */
    ckeditorInit(config) {
        if (typeof CKEDITOR === 'undefined') {
            console.warn('CKEditor 4.x is missing (http://ckeditor.com/)');
        }
        else {
            // Check textarea exists
            if (this.instance || !this.documentContains(this.host.nativeElement)) {
                return;
            }
            if (this.readonly) {
                config.readOnly = this.readonly;
            }
            // CKEditor replace textarea
            this.instance = CKEDITOR.replace(this.host.nativeElement, config);
            // Set initial value
            this.instance.setData(this.value);
            // listen for instanceReady event
            this.instance.on('instanceReady', (evt) => {
                // if value has changed while instance loading
                // update instance with current component value
                if (this.instance.getData() !== this.value) {
                    this.instance.setData(this.value);
                }
                // send the evt to the EventEmitter
                this.ready.emit(evt);
            });
            // CKEditor change event
            this.instance.on('change', (evt) => {
                // console.log(evt);
                this.onTouched();
                let /** @type {?} */ value = this.instance.getData();
                if (this.value !== value) {
                    // Debounce update
                    if (this.debounce) {
                        if (this.debounceTimeout) {
                            clearTimeout(this.debounceTimeout);
                        }
                        this.debounceTimeout = setTimeout(() => {
                            this.updateValue(value);
                            this.debounceTimeout = null;
                        }, parseInt(this.debounce, 10));
                        // Live update
                    }
                    else {
                        this.updateValue(value);
                    }
                }
                // Original ckeditor event dispatch
                this.editorChange.emit(evt);
            });
            this.instance.on('contentDom', () => {
                let /** @type {?} */ editable = this.instance.editable();
                editable.attachListener(this.instance.document, 'compositionend', (evt) => {
                    this.onTouched();
                    let /** @type {?} */ value = this.instance.getData();
                    if (this.value !== value) {
                        // Debounce update
                        if (this.debounce) {
                            if (this.debounceTimeout) {
                                clearTimeout(this.debounceTimeout);
                            }
                            this.debounceTimeout = setTimeout(() => {
                                this.updateValue(value);
                                this.debounceTimeout = null;
                            }, parseInt(this.debounce, 10));
                            // Live update
                        }
                        else {
                            this.updateValue(value);
                        }
                    }
                    // Original ckeditor event dispatch
                    this.editorChange.emit(evt);
                });
            });
            // CKEditor blur event
            this.instance.on('blur', (evt) => {
                this.blur.emit(evt);
            });
            // CKEditor focus event
            this.instance.on('focus', (evt) => {
                this.focus.emit(evt);
            });
            // CKEditor contentDom event
            this.instance.on('contentDom', (evt) => {
                this.contentDom.emit(evt);
            });
            // CKEditor fileUploadRequest event
            this.instance.on('fileUploadRequest', (evt) => {
                this.fileUploadRequest.emit(evt);
            });
            // Add Toolbar Groups to Editor. This will also add Buttons within groups.
            this.toolbarGroups.forEach(group => {
                group.initialize(this);
            });
            // Add Toolbar Buttons to Editor.
            this.toolbarButtons.forEach(button => {
                button.initialize(this);
            });
        }
    }
    /**
     * Implements ControlValueAccessor
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._value = value;
        if (this.instance) {
            this.instance.setData(value);
        }
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onChange(_) { }
    /**
     * @return {?}
     */
    onTouched() { }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} node
     * @return {?}
     */
    documentContains(node) {
        return document.contains ? document.contains(node) : document.body.contains(node);
    }
}
CKEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ckeditor, cui-ckeditor',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => CKEditorComponent),
                        multi: true,
                    },
                ],
                template: `<textarea #host></textarea>`,
            },] },
];
/**
 * @nocollapse
 */
CKEditorComponent.ctorParameters = () => [
    { type: NgZone, },
];
CKEditorComponent.propDecorators = {
    'readonly': [{ type: Input },],
    'debounce': [{ type: Input },],
    'toolbarMode': [{ type: Input },],
    'urlWithBasePath': [{ type: Input },],
    'change': [{ type: Output },],
    'editorChange': [{ type: Output },],
    'ready': [{ type: Output },],
    'blur': [{ type: Output },],
    'focus': [{ type: Output },],
    'contentDom': [{ type: Output },],
    'fileUploadRequest': [{ type: Output },],
    'host': [{ type: ViewChild, args: ['host',] },],
    'toolbarButtons': [{ type: ContentChildren, args: [CKButtonDirective,] },],
    'toolbarGroups': [{ type: ContentChildren, args: [CKGroupDirective,] },],
    'config': [{ type: Input },],
    'value': [{ type: Input },],
};
function CKEditorComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    CKEditorComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    CKEditorComponent.ctorParameters;
    /** @type {?} */
    CKEditorComponent.propDecorators;
    /** @type {?} */
    CKEditorComponent.prototype.readonly;
    /** @type {?} */
    CKEditorComponent.prototype.debounce;
    /** @type {?} */
    CKEditorComponent.prototype.toolbarMode;
    /** @type {?} */
    CKEditorComponent.prototype.urlWithBasePath;
    /** @type {?} */
    CKEditorComponent.prototype.change;
    /** @type {?} */
    CKEditorComponent.prototype.editorChange;
    /** @type {?} */
    CKEditorComponent.prototype.ready;
    /** @type {?} */
    CKEditorComponent.prototype.blur;
    /** @type {?} */
    CKEditorComponent.prototype.focus;
    /** @type {?} */
    CKEditorComponent.prototype.contentDom;
    /** @type {?} */
    CKEditorComponent.prototype.fileUploadRequest;
    /** @type {?} */
    CKEditorComponent.prototype.host;
    /** @type {?} */
    CKEditorComponent.prototype.toolbarButtons;
    /** @type {?} */
    CKEditorComponent.prototype.toolbarGroups;
    /** @type {?} */
    CKEditorComponent.prototype._value;
    /** @type {?} */
    CKEditorComponent.prototype.instance;
    /** @type {?} */
    CKEditorComponent.prototype.debounceTimeout;
    /** @type {?} */
    CKEditorComponent.prototype.defaultConfig;
    /** @type {?} */
    CKEditorComponent.prototype._config;
    /** @type {?} */
    CKEditorComponent.prototype.zone;
}
