import { Directive, Input, ContentChildren } from '@angular/core';
import { CKButtonDirective } from './ckbutton.directive';
/**
 * CKGroup component
 * Usage :
 *  <ckeditor [(ngModel)]="data" [config]="{...}" debounce="500">
 *      <ckgroup [name]="'exampleGroup2'" [previous]="'1'" [subgroupOf]="'exampleGroup1'">
 *          .
 *          .
 *      </ckgroup>
 *   </ckeditor>
 */
export class CKGroupDirective {
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        // Reconfigure each button's toolbar property within ckgroup to hold its parent's name
        this.toolbarButtons.forEach(button => (button.toolbar = this.name));
    }
    /**
     * @param {?} editor
     * @return {?}
     */
    initialize(editor) {
        editor.instance.ui.addToolbarGroup(this.name, this.previous, this.subgroupOf);
        // Initialize each button within ckgroup
        this.toolbarButtons.forEach(button => {
            button.initialize(editor);
        });
    }
}
CKGroupDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ckgroup',
            },] },
];
/**
 * @nocollapse
 */
CKGroupDirective.ctorParameters = () => [];
CKGroupDirective.propDecorators = {
    'name': [{ type: Input },],
    'previous': [{ type: Input },],
    'subgroupOf': [{ type: Input },],
    'toolbarButtons': [{ type: ContentChildren, args: [CKButtonDirective,] },],
};
function CKGroupDirective_tsickle_Closure_declarations() {
    /** @type {?} */
    CKGroupDirective.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    CKGroupDirective.ctorParameters;
    /** @type {?} */
    CKGroupDirective.propDecorators;
    /** @type {?} */
    CKGroupDirective.prototype.name;
    /** @type {?} */
    CKGroupDirective.prototype.previous;
    /** @type {?} */
    CKGroupDirective.prototype.subgroupOf;
    /** @type {?} */
    CKGroupDirective.prototype.toolbarButtons;
}
