import { Component, ElementRef, Input, forwardRef, EventEmitter, Output } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomHandler } from '../../core/dom/dom-handler';
export const /** @type {?} */ INPUTSWITCH_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => InputSwitchComponent),
    multi: true
};
export class InputSwitchComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     */
    constructor(el, domHandler) {
        this.el = el;
        this.domHandler = domHandler;
        this.onLabel = 'On';
        this.offLabel = 'Off';
        this.ariaLabelTemplate = "InputSwitch {0}";
        this.onChange = new EventEmitter();
        this.checked = false;
        this.focused = false;
        this.initialized = false;
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.container = this.el.nativeElement.children[0];
        this.handle = this.domHandler.findSingle(this.el.nativeElement, 'div.ui-inputswitch-handle');
        this.onContainer = this.domHandler.findSingle(this.container, 'div.ui-inputswitch-on');
        this.offContainer = this.domHandler.findSingle(this.container, 'div.ui-inputswitch-off');
        this.onLabelChild = this.domHandler.findSingle(this.onContainer, 'span.ui-inputswitch-onlabel');
        this.offLabelChild = this.domHandler.findSingle(this.offContainer, 'span.ui-inputswitch-offlabel');
        this.setAriaLabel();
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        if (this.container.offsetParent && !this.initialized) {
            this.render();
        }
    }
    /**
     * @return {?}
     */
    render() {
        let /** @type {?} */ onContainerWidth = this.domHandler.width(this.onContainer), /** @type {?} */ offContainerWidth = this.domHandler.width(this.offContainer), /** @type {?} */ spanPadding = this.domHandler.innerWidth(this.offLabelChild) - this.domHandler.width(this.offLabelChild), /** @type {?} */ handleMargins = this.domHandler.getOuterWidth(this.handle) - this.domHandler.innerWidth(this.handle);
        let /** @type {?} */ containerWidth = (onContainerWidth > offContainerWidth) ? onContainerWidth : offContainerWidth, /** @type {?} */ handleWidth = containerWidth;
        this.handle.style.width = handleWidth + 'px';
        handleWidth = this.domHandler.width(this.handle);
        containerWidth = containerWidth + handleWidth + 6;
        let /** @type {?} */ labelWidth = containerWidth - handleWidth - spanPadding - handleMargins;
        this.container.style.width = containerWidth + 'px';
        this.onLabelChild.style.width = labelWidth + 'px';
        this.offLabelChild.style.width = labelWidth + 'px';
        // position
        this.offContainer.style.width = (this.domHandler.width(this.container) - 5) + 'px';
        this.offset = this.domHandler.width(this.container) - this.domHandler.getOuterWidth(this.handle);
        // default value
        if (this.checked) {
            this.handle.style.left = this.offset + 'px';
            this.onContainer.style.width = this.offset + 'px';
            this.offLabelChild.style.marginRight = -this.offset + 'px';
        }
        else {
            this.onContainer.style.width = 0 + 'px';
            this.onLabelChild.style.marginLeft = -this.offset + 'px';
        }
        this.initialized = true;
    }
    /**
     * @param {?} event
     * @param {?} checkbox
     * @return {?}
     */
    toggle(event, checkbox) {
        if (!this.disabled) {
            if (this.checked) {
                this.checked = false;
                this.uncheckUI();
            }
            else {
                this.checked = true;
                this.checkUI();
            }
            this.setAriaLabel();
            this.onModelChange(this.checked);
            this.onChange.emit({
                originalEvent: event,
                checked: this.checked
            });
            checkbox.focus();
        }
    }
    /**
     * @return {?}
     */
    checkUI() {
        this.onContainer.style.width = this.offset + 'px';
        this.onLabelChild.style.marginLeft = 0 + 'px';
        this.offLabelChild.style.marginRight = -this.offset + 'px';
        this.handle.style.left = this.offset + 'px';
    }
    /**
     * @return {?}
     */
    uncheckUI() {
        this.onContainer.style.width = 0 + 'px';
        this.onLabelChild.style.marginLeft = -this.offset + 'px';
        this.offLabelChild.style.marginRight = 0 + 'px';
        this.handle.style.left = 0 + 'px';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        this.focused = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBlur(event) {
        this.focused = false;
        this.onModelTouched();
    }
    /**
     * @param {?} checked
     * @return {?}
     */
    writeValue(checked) {
        this.checked = checked;
        if (this.initialized) {
            if (this.checked === true) {
                this.checkUI();
            }
            else {
                this.uncheckUI();
            }
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setDisabledState(val) {
        this.disabled = val;
    }
    /**
     * @return {?}
     */
    setAriaLabel() {
        let /** @type {?} */ pattern = /{(.*?)}/, /** @type {?} */ value = this.checked ? this.onLabel : this.offLabel;
        this.ariaLabel = this.ariaLabelTemplate.replace(this.ariaLabelTemplate.match(pattern)[0], value);
    }
}
InputSwitchComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-input-switch',
                templateUrl: './input-switch.component.html',
                styleUrls: ['./input-switch.component.scss'],
                providers: [INPUTSWITCH_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
InputSwitchComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
];
InputSwitchComponent.propDecorators = {
    'onLabel': [{ type: Input },],
    'offLabel': [{ type: Input },],
    'disabled': [{ type: Input },],
    'style': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'tabindex': [{ type: Input },],
    'inputId': [{ type: Input },],
    'ariaLabelTemplate': [{ type: Input },],
    'onChange': [{ type: Output },],
};
function InputSwitchComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    InputSwitchComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    InputSwitchComponent.ctorParameters;
    /** @type {?} */
    InputSwitchComponent.propDecorators;
    /** @type {?} */
    InputSwitchComponent.prototype.onLabel;
    /** @type {?} */
    InputSwitchComponent.prototype.offLabel;
    /** @type {?} */
    InputSwitchComponent.prototype.disabled;
    /** @type {?} */
    InputSwitchComponent.prototype.style;
    /** @type {?} */
    InputSwitchComponent.prototype.styleClass;
    /** @type {?} */
    InputSwitchComponent.prototype.tabindex;
    /** @type {?} */
    InputSwitchComponent.prototype.inputId;
    /** @type {?} */
    InputSwitchComponent.prototype.ariaLabelTemplate;
    /** @type {?} */
    InputSwitchComponent.prototype.onChange;
    /** @type {?} */
    InputSwitchComponent.prototype.checked;
    /** @type {?} */
    InputSwitchComponent.prototype.focused;
    /** @type {?} */
    InputSwitchComponent.prototype.container;
    /** @type {?} */
    InputSwitchComponent.prototype.handle;
    /** @type {?} */
    InputSwitchComponent.prototype.onContainer;
    /** @type {?} */
    InputSwitchComponent.prototype.offContainer;
    /** @type {?} */
    InputSwitchComponent.prototype.onLabelChild;
    /** @type {?} */
    InputSwitchComponent.prototype.offLabelChild;
    /** @type {?} */
    InputSwitchComponent.prototype.offset;
    /** @type {?} */
    InputSwitchComponent.prototype.ariaLabel;
    /** @type {?} */
    InputSwitchComponent.prototype.ariaLabelledBy;
    /** @type {?} */
    InputSwitchComponent.prototype.initialized;
    /** @type {?} */
    InputSwitchComponent.prototype.onModelChange;
    /** @type {?} */
    InputSwitchComponent.prototype.onModelTouched;
    /** @type {?} */
    InputSwitchComponent.prototype.el;
    /** @type {?} */
    InputSwitchComponent.prototype.domHandler;
}
