import { Component, forwardRef, ElementRef, NgZone, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
const /** @type {?} */ UEDITOR_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => UeditorComponent),
    multi: true
};
export class UeditorComponent {
    /**
     * @param {?} el
     * @param {?} ngZone
     */
    constructor(el, ngZone) {
        this.el = el;
        this.ngZone = ngZone;
        this.umToolbar = [
            ['source', '|', 'undo', 'redo', '|', 'bold', 'italic', 'emotion', '|', 'underline',
                'strikethrough', '|', 'superscript', 'subscript', '|', 'forecolor',
                'backcolor', '|', 'removeformat', '|',
                'insertorderedlist', 'insertunorderedlist', '|', 'selectall', 'cleardoc', 'paragraph', '|', 'fontfamily', 'fontsize',
                '|', 'justifyleft', 'justifycenter', 'justifyright', 'justifyjustify', '|',
                'link', 'unlink', '|', 'simpleupload', 'insertvideo', 'attachment']
        ];
        this.autoHeight = false;
        this.initialFrameHeight = 450;
        this.topOffset = 0;
        this.fontsize = 14;
        this.toolbarMode = "default";
        this.onReady = new EventEmitter();
        this.onValueChange = new EventEmitter();
        this.onFocus = new EventEmitter();
        this.onChange = () => { };
        this.onTouched = () => { };
    }
    /**
     * @return {?}
     */
    ueditorSettings() {
        let /** @type {?} */ defaultConfig = {
            autoHeightEnabled: this.autoHeight,
            initialFrameHeight: this.initialFrameHeight,
            topOffset: this.topOffset,
            elementPathEnabled: false,
        };
        if (this.toolbarMode === 'um') {
            // debugger;
            defaultConfig['toolbars'] = this.umToolbar;
            defaultConfig['wordCount'] = false;
            defaultConfig['elementPathEnabled'] = false;
        }
        let /** @type {?} */ cf = Object.assign(defaultConfig, this.setting);
        return cf;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.init();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.value = value;
        if (this.ueditor) {
            this.ueditor.setContent(this.value);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        if (isDisabled) {
            this.ueditor.setDisabled();
        }
        else {
            this.ueditor.setEnabled();
        }
    }
    /**
     * @return {?}
     */
    init() {
        if (typeof UE === 'undefined') {
            console.error('UEditor is missing');
            return;
        }
        let /** @type {?} */ ueditor = new UE.ui.Editor(this.ueditorSettings() || {});
        ueditor.render(this.host.nativeElement);
        ueditor.addListener('ready', () => {
            this.ueditor = ueditor;
            this.ueditor.execCommand('fontsize', this.fontsize + 'px');
            this.ueditor.execCommand('fontfamily', '微软雅黑');
            if (this.value) {
                this.ueditor.setContent(this.value);
            }
            this.onReady.emit();
            this.addEvent();
        });
    }
    /**
     * @return {?}
     */
    addEvent() {
        this.ueditor.addListener('contentChange', () => {
            this.updateValue(this.ueditor.getContent());
        });
        this.ueditor.addListener('focus', () => {
            this.onFocus.emit();
        });
    }
    /**
     * @return {?}
     */
    destroy() {
        if (this.ueditor) {
            this.ueditor.removeListener('ready');
            this.ueditor.removeListener('contentChange');
            this.ueditor.destroy();
            this.ueditor = null;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.ngZone.run(() => {
            this.value = value;
            this.onChange(this.value);
            this.onTouched();
            this.onValueChange.emit(this.value);
        });
    }
    /**
     * @return {?}
     */
    getContent() {
        return this.ueditor.getContent();
    }
    /**
     * @return {?}
     */
    getPlainTxt() {
        return this.ueditor.getPlainTxt();
    }
    /**
     * @return {?}
     */
    getContentTxt() {
        return this.ueditor.getContentTxt();
    }
    /**
     * @return {?}
     */
    hasContents() {
        return this.ueditor.hasContents();
    }
}
UeditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-ueditor',
                template: '<textarea #host></textarea>',
                providers: [UEDITOR_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
UeditorComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
];
UeditorComponent.propDecorators = {
    'setting': [{ type: Input },],
    'autoHeight': [{ type: Input },],
    'initialFrameHeight': [{ type: Input },],
    'topOffset': [{ type: Input },],
    'fontsize': [{ type: Input },],
    'toolbarMode': [{ type: Input },],
    'onReady': [{ type: Output },],
    'onValueChange': [{ type: Output },],
    'onFocus': [{ type: Output },],
    'host': [{ type: ViewChild, args: ['host',] },],
};
function UeditorComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    UeditorComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    UeditorComponent.ctorParameters;
    /** @type {?} */
    UeditorComponent.propDecorators;
    /** @type {?} */
    UeditorComponent.prototype.ueditor;
    /** @type {?} */
    UeditorComponent.prototype.value;
    /** @type {?} */
    UeditorComponent.prototype.umToolbar;
    /** @type {?} */
    UeditorComponent.prototype.setting;
    /** @type {?} */
    UeditorComponent.prototype.autoHeight;
    /** @type {?} */
    UeditorComponent.prototype.initialFrameHeight;
    /** @type {?} */
    UeditorComponent.prototype.topOffset;
    /** @type {?} */
    UeditorComponent.prototype.fontsize;
    /** @type {?} */
    UeditorComponent.prototype.toolbarMode;
    /** @type {?} */
    UeditorComponent.prototype.onReady;
    /** @type {?} */
    UeditorComponent.prototype.onValueChange;
    /** @type {?} */
    UeditorComponent.prototype.onFocus;
    /** @type {?} */
    UeditorComponent.prototype.host;
    /** @type {?} */
    UeditorComponent.prototype.onChange;
    /** @type {?} */
    UeditorComponent.prototype.onTouched;
    /** @type {?} */
    UeditorComponent.prototype.el;
    /** @type {?} */
    UeditorComponent.prototype.ngZone;
}
