import { Component, Input, ContentChildren, Output, EventEmitter } from '@angular/core';
import { WizardStepComponent } from './wizard-step/wizard-step.component';
export class WizardComponent {
    constructor() {
        this.direction = 'horizontal'; // horizontal | vertical
        this.prevClick = new EventEmitter();
        this.nextClick = new EventEmitter();
        this.finishClick = new EventEmitter();
        this.cancelClick = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!(this.activeIndex > 0)) {
            this.activeIndex = 0;
        }
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        console.log('Wizard inited.');
        this.steps.forEach((it, i) => {
            if (!it.setpIndex) {
                it.setpIndex = i;
            }
        });
        this.refreshActive();
    }
    /**
     * @param {?} event
     * @param {?} item
     * @param {?} i
     * @return {?}
     */
    itemClick(event, item, i) {
    }
    /**
     * @return {?}
     */
    prev() {
        if (this.prevEnabled) {
            this.activeIndex--;
        }
        this.refreshActive();
        this.prevClick.emit(this.activeIndex);
    }
    /**
     * @return {?}
     */
    next() {
        if (this.nextEnabled && this.activeStep.valid) {
            this.activeIndex++;
        }
        this.refreshActive();
        this.nextClick.emit(this.activeIndex);
    }
    /**
     * @return {?}
     */
    finish() {
        if (this.finishEnabled && this.activeStep.valid) {
            console.log("wizard finished");
            this.finishClick.emit();
        }
    }
    /**
     * @return {?}
     */
    cancel() {
        if (this.cancelEnabled) {
            console.log("wizard canceled");
            this.cancelClick.emit();
        }
    }
    /**
     * @return {?}
     */
    refreshActive() {
        if (!(this.activeIndex > 0)) {
            this.activeIndex = 0;
        }
        this.steps.forEach((it, i) => {
            it.complete = i < this.activeIndex;
            it.active = it.setpIndex == this.activeIndex;
        });
        this.activeStep = this.steps.find(it => it.setpIndex == this.activeIndex);
        this.prevEnabled = this.activeIndex > 0;
        this.nextEnabled = this.activeIndex < this.steps.length - 1;
        this.finishEnabled = this.activeIndex == this.steps.length - 1;
        this.cancelEnabled = true;
    }
}
WizardComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-wizard',
                templateUrl: './wizard.component.html',
                styleUrls: ['./wizard.component.scss']
            },] },
];
/**
 * @nocollapse
 */
WizardComponent.ctorParameters = () => [];
WizardComponent.propDecorators = {
    'direction': [{ type: Input },],
    'steps': [{ type: ContentChildren, args: [WizardStepComponent,] },],
    'prevClick': [{ type: Output },],
    'nextClick': [{ type: Output },],
    'finishClick': [{ type: Output },],
    'cancelClick': [{ type: Output },],
};
function WizardComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    WizardComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    WizardComponent.ctorParameters;
    /** @type {?} */
    WizardComponent.propDecorators;
    /** @type {?} */
    WizardComponent.prototype.direction;
    /** @type {?} */
    WizardComponent.prototype.steps;
    /** @type {?} */
    WizardComponent.prototype.prevClick;
    /** @type {?} */
    WizardComponent.prototype.nextClick;
    /** @type {?} */
    WizardComponent.prototype.finishClick;
    /** @type {?} */
    WizardComponent.prototype.cancelClick;
    /** @type {?} */
    WizardComponent.prototype.activeIndex;
    /** @type {?} */
    WizardComponent.prototype.activeStep;
    /** @type {?} */
    WizardComponent.prototype.prevEnabled;
    /** @type {?} */
    WizardComponent.prototype.nextEnabled;
    /** @type {?} */
    WizardComponent.prototype.finishEnabled;
    /** @type {?} */
    WizardComponent.prototype.cancelEnabled;
}
