import { Injectable } from '@angular/core';
export class DomHandler {
    constructor() {
        this.calculatedScrollbarWidth = null;
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    addClass(element, className) {
        if (element.classList)
            element.classList.add(className);
        else
            element.className += ' ' + className;
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    addMultipleClasses(element, className) {
        if (element.classList) {
            let /** @type {?} */ styles = className.split(' ');
            for (let /** @type {?} */ i = 0; i < styles.length; i++) {
                element.classList.add(styles[i]);
            }
        }
        else {
            let /** @type {?} */ styles = className.split(' ');
            for (let /** @type {?} */ i = 0; i < styles.length; i++) {
                element.className += ' ' + styles[i];
            }
        }
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    removeClass(element, className) {
        if (element.classList)
            element.classList.remove(className);
        else
            element.className = element.className.replace(new RegExp('(^|\\b)' + className.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    hasClass(element, className) {
        if (element.classList)
            return element.classList.contains(className);
        else
            return new RegExp('(^| )' + className + '( |$)', 'gi').test(element.className);
    }
    /**
     * @param {?} element
     * @return {?}
     */
    siblings(element) {
        return Array.prototype.filter.call(element.parentNode.children, function (child) {
            return child !== element;
        });
    }
    /**
     * @param {?} element
     * @param {?} selector
     * @return {?}
     */
    find(element, selector) {
        return element.querySelectorAll(selector);
    }
    /**
     * @param {?} element
     * @param {?} selector
     * @return {?}
     */
    findSingle(element, selector) {
        return element.querySelector(selector);
    }
    /**
     * @param {?} element
     * @return {?}
     */
    index(element) {
        let /** @type {?} */ children = element.parentNode.childNodes;
        let /** @type {?} */ num = 0;
        for (var /** @type {?} */ i = 0; i < children.length; i++) {
            if (children[i] == element)
                return num;
            if (children[i].nodeType == 1)
                num++;
        }
        return -1;
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    relativePosition(element, target) {
        let /** @type {?} */ elementDimensions = element.offsetParent ? { width: element.offsetWidth, height: element.offsetHeight } : this.getHiddenElementDimensions(element);
        let /** @type {?} */ targetHeight = target.offsetHeight;
        let /** @type {?} */ targetWidth = target.offsetWidth;
        let /** @type {?} */ targetOffset = target.getBoundingClientRect();
        let /** @type {?} */ windowScrollTop = this.getWindowScrollTop();
        let /** @type {?} */ viewport = this.getViewport();
        let /** @type {?} */ top, /** @type {?} */ left;
        if ((targetOffset.top + targetHeight + elementDimensions.height) > viewport.height) {
            top = -1 * (elementDimensions.height);
            if (targetOffset.top + top < 0) {
                top = 0;
            }
        }
        else {
            top = targetHeight;
        }
        if ((targetOffset.left + elementDimensions.width) > viewport.width)
            left = targetWidth - elementDimensions.width;
        else
            left = 0;
        element.style.top = top + 'px';
        element.style.left = left + 'px';
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    absolutePosition(element, target) {
        let /** @type {?} */ elementDimensions = element.offsetParent ? { width: element.offsetWidth, height: element.offsetHeight } : this.getHiddenElementDimensions(element);
        let /** @type {?} */ elementOuterHeight = elementDimensions.height;
        let /** @type {?} */ elementOuterWidth = elementDimensions.width;
        let /** @type {?} */ targetOuterHeight = target.offsetHeight;
        let /** @type {?} */ targetOuterWidth = target.offsetWidth;
        let /** @type {?} */ targetOffset = target.getBoundingClientRect();
        let /** @type {?} */ windowScrollTop = this.getWindowScrollTop();
        let /** @type {?} */ windowScrollLeft = this.getWindowScrollLeft();
        let /** @type {?} */ viewport = this.getViewport();
        let /** @type {?} */ top, /** @type {?} */ left;
        if (targetOffset.top + targetOuterHeight + elementOuterHeight > viewport.height) {
            top = targetOffset.top + windowScrollTop - elementOuterHeight;
            if (top < 0) {
                top = 0 + windowScrollTop;
            }
        }
        else {
            top = targetOuterHeight + targetOffset.top + windowScrollTop;
        }
        if (targetOffset.left + targetOuterWidth + elementOuterWidth > viewport.width)
            left = targetOffset.left + windowScrollLeft + targetOuterWidth - elementOuterWidth;
        else
            left = targetOffset.left + windowScrollLeft;
        element.style.top = top + 'px';
        element.style.left = left + 'px';
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getHiddenElementOuterHeight(element) {
        element.style.visibility = 'hidden';
        element.style.display = 'block';
        let /** @type {?} */ elementHeight = element.offsetHeight;
        element.style.display = 'none';
        element.style.visibility = 'visible';
        return elementHeight;
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getHiddenElementOuterWidth(element) {
        element.style.visibility = 'hidden';
        element.style.display = 'block';
        let /** @type {?} */ elementWidth = element.offsetWidth;
        element.style.display = 'none';
        element.style.visibility = 'visible';
        return elementWidth;
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getHiddenElementDimensions(element) {
        let /** @type {?} */ dimensions = {};
        element.style.visibility = 'hidden';
        element.style.display = 'block';
        dimensions.width = element.offsetWidth;
        dimensions.height = element.offsetHeight;
        element.style.display = 'none';
        element.style.visibility = 'visible';
        return dimensions;
    }
    /**
     * @param {?} container
     * @param {?} item
     * @return {?}
     */
    scrollInView(container, item) {
        let /** @type {?} */ borderTopValue = getComputedStyle(container).getPropertyValue('borderTopWidth');
        let /** @type {?} */ borderTop = borderTopValue ? parseFloat(borderTopValue) : 0;
        let /** @type {?} */ paddingTopValue = getComputedStyle(container).getPropertyValue('paddingTop');
        let /** @type {?} */ paddingTop = paddingTopValue ? parseFloat(paddingTopValue) : 0;
        let /** @type {?} */ containerRect = container.getBoundingClientRect();
        let /** @type {?} */ itemRect = item.getBoundingClientRect();
        let /** @type {?} */ offset = (itemRect.top + document.body.scrollTop) - (containerRect.top + document.body.scrollTop) - borderTop - paddingTop;
        let /** @type {?} */ scroll = container.scrollTop;
        let /** @type {?} */ elementHeight = container.clientHeight;
        let /** @type {?} */ itemHeight = this.getOuterHeight(item);
        if (offset < 0) {
            container.scrollTop = scroll + offset;
        }
        else if ((offset + itemHeight) > elementHeight) {
            container.scrollTop = scroll + offset - elementHeight + itemHeight;
        }
    }
    /**
     * @param {?} element
     * @param {?} duration
     * @return {?}
     */
    fadeIn(element, duration) {
        element.style.opacity = 0;
        let /** @type {?} */ last = +new Date();
        let /** @type {?} */ opacity = 0;
        let /** @type {?} */ tick = function () {
            opacity = +element.style.opacity + (new Date().getTime() - last) / duration;
            element.style.opacity = opacity;
            last = +new Date();
            if (+opacity < 1) {
                (window.requestAnimationFrame && requestAnimationFrame(tick)) || setTimeout(tick, 16);
            }
        };
        tick();
    }
    /**
     * @param {?} element
     * @param {?} ms
     * @return {?}
     */
    fadeOut(element, ms) {
        var /** @type {?} */ opacity = 1, /** @type {?} */ interval = 50, /** @type {?} */ duration = ms, /** @type {?} */ gap = interval / duration;
        let /** @type {?} */ fading = setInterval(() => {
            opacity = opacity - gap;
            if (opacity <= 0) {
                opacity = 0;
                clearInterval(fading);
            }
            element.style.opacity = opacity;
        }, interval);
    }
    /**
     * @return {?}
     */
    getWindowScrollTop() {
        let /** @type {?} */ doc = document.documentElement;
        return (window.pageYOffset || doc.scrollTop) - (doc.clientTop || 0);
    }
    /**
     * @return {?}
     */
    getWindowScrollLeft() {
        let /** @type {?} */ doc = document.documentElement;
        return (window.pageXOffset || doc.scrollLeft) - (doc.clientLeft || 0);
    }
    /**
     * @param {?} element
     * @param {?} selector
     * @return {?}
     */
    matches(element, selector) {
        var /** @type {?} */ p = Element.prototype;
        var /** @type {?} */ f = p['matches'] || p.webkitMatchesSelector || p['mozMatchesSelector'] || p.msMatchesSelector || function (s) {
            return [].indexOf.call(document.querySelectorAll(s), this) !== -1;
        };
        return f.call(element, selector);
    }
    /**
     * @param {?} el
     * @param {?=} margin
     * @return {?}
     */
    getOuterWidth(el, margin) {
        let /** @type {?} */ width = el.offsetWidth;
        if (margin) {
            let /** @type {?} */ style = getComputedStyle(el);
            width += parseFloat(style.marginLeft) + parseFloat(style.marginRight);
        }
        return width;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getHorizontalPadding(el) {
        let /** @type {?} */ style = getComputedStyle(el);
        return parseFloat(style.paddingLeft) + parseFloat(style.paddingRight);
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getHorizontalMargin(el) {
        let /** @type {?} */ style = getComputedStyle(el);
        return parseFloat(style.marginLeft) + parseFloat(style.marginRight);
    }
    /**
     * @param {?} el
     * @return {?}
     */
    innerWidth(el) {
        let /** @type {?} */ width = el.offsetWidth;
        let /** @type {?} */ style = getComputedStyle(el);
        width += parseFloat(style.paddingLeft) + parseFloat(style.paddingRight);
        return width;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    width(el) {
        let /** @type {?} */ width = el.offsetWidth;
        let /** @type {?} */ style = getComputedStyle(el);
        width -= parseFloat(style.paddingLeft) + parseFloat(style.paddingRight);
        return width;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getInnerHeight(el) {
        let /** @type {?} */ height = el.offsetHeight;
        let /** @type {?} */ style = getComputedStyle(el);
        height += parseFloat(style.paddingTop) + parseFloat(style.paddingBottom);
        return height;
    }
    /**
     * @param {?} el
     * @param {?=} margin
     * @return {?}
     */
    getOuterHeight(el, margin) {
        let /** @type {?} */ height = el.offsetHeight;
        if (margin) {
            let /** @type {?} */ style = getComputedStyle(el);
            height += parseFloat(style.marginTop) + parseFloat(style.marginBottom);
        }
        return height;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getHeight(el) {
        let /** @type {?} */ height = el.offsetHeight;
        let /** @type {?} */ style = getComputedStyle(el);
        height -= parseFloat(style.paddingTop) + parseFloat(style.paddingBottom) + parseFloat(style.borderTopWidth) + parseFloat(style.borderBottomWidth);
        return height;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getWidth(el) {
        let /** @type {?} */ width = el.offsetWidth;
        let /** @type {?} */ style = getComputedStyle(el);
        width -= parseFloat(style.paddingLeft) + parseFloat(style.paddingRight) + parseFloat(style.borderLeftWidth) + parseFloat(style.borderRightWidth);
        return width;
    }
    /**
     * @return {?}
     */
    getViewport() {
        let /** @type {?} */ win = window, /** @type {?} */ d = document, /** @type {?} */ e = d.documentElement, /** @type {?} */ g = d.getElementsByTagName('body')[0], /** @type {?} */ w = win.innerWidth || e.clientWidth || g.clientWidth, /** @type {?} */ h = win.innerHeight || e.clientHeight || g.clientHeight;
        return { width: w, height: h };
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getOffset(el) {
        let /** @type {?} */ x = el.offsetLeft;
        let /** @type {?} */ y = el.offsetTop;
        while (el = el.offsetParent) {
            x += el.offsetLeft;
            y += el.offsetTop;
        }
        return { left: x, top: y };
    }
    /**
     * @return {?}
     */
    getUserAgent() {
        return navigator.userAgent;
    }
    /**
     * @return {?}
     */
    isIE() {
        var /** @type {?} */ ua = window.navigator.userAgent;
        var /** @type {?} */ msie = ua.indexOf('MSIE ');
        if (msie > 0) {
            // IE 10 or older => return version number
            return true;
        }
        var /** @type {?} */ trident = ua.indexOf('Trident/');
        if (trident > 0) {
            // IE 11 => return version number
            var /** @type {?} */ rv = ua.indexOf('rv:');
            return true;
        }
        var /** @type {?} */ edge = ua.indexOf('Edge/');
        if (edge > 0) {
            // Edge (IE 12+) => return version number
            return true;
        }
        // other browser
        return false;
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    appendChild(element, target) {
        if (this.isElement(target))
            target.appendChild(element);
        else if (target.el && target.el.nativeElement)
            target.el.nativeElement.appendChild(element);
        else
            throw 'Cannot append ' + target + ' to ' + element;
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    removeChild(element, target) {
        if (this.isElement(target))
            target.removeChild(element);
        else if (target.el && target.el.nativeElement)
            target.el.nativeElement.removeChild(element);
        else
            throw 'Cannot remove ' + element + ' from ' + target;
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    isElement(obj) {
        return (typeof HTMLElement === "object" ? obj instanceof HTMLElement :
            obj && typeof obj === "object" && obj !== null && obj.nodeType === 1 && typeof obj.nodeName === "string");
    }
    /**
     * @return {?}
     */
    calculateScrollbarWidth() {
        if (this.calculatedScrollbarWidth !== null)
            return this.calculatedScrollbarWidth;
        let /** @type {?} */ scrollDiv = document.createElement("div");
        scrollDiv.className = "ui-scrollbar-measure";
        document.body.appendChild(scrollDiv);
        let /** @type {?} */ scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
        document.body.removeChild(scrollDiv);
        this.calculatedScrollbarWidth = scrollbarWidth;
        return scrollbarWidth;
    }
    /**
     * @param {?} element
     * @param {?} methodName
     * @param {?=} args
     * @return {?}
     */
    invokeElementMethod(element, methodName, args) {
        ((element))[methodName].apply(element, args);
    }
}
DomHandler.zindex = 1000;
DomHandler.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
DomHandler.ctorParameters = () => [];
function DomHandler_tsickle_Closure_declarations() {
    /** @type {?} */
    DomHandler.zindex;
    /** @type {?} */
    DomHandler.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    DomHandler.ctorParameters;
    /** @type {?} */
    DomHandler.prototype.calculatedScrollbarWidth;
}
