import { Injectable, ComponentFactoryResolver, ApplicationRef, Injector, NgZone } from '@angular/core';
import { OverlayContainer, OVERLAY_CONTAINER_PROVIDER } from './overlay-container';
import { OverlayRef } from './overlay-ref';
import { DomPortalHost } from '../portal/dom-portal-host';
import { OverlayPositionBuilder } from './position/overlay-position-builder';
let /** @type {?} */ nextUniqueId = 0;
export class Overlay {
    /**
     * @param {?} overlayContainer
     * @param {?} postionBuilder
     * @param {?} componentFactoryResolver
     * @param {?} appRef
     * @param {?} injector
     * @param {?} ngZone
     */
    constructor(overlayContainer, postionBuilder, componentFactoryResolver, appRef, injector, ngZone) {
        this.overlayContainer = overlayContainer;
        this.postionBuilder = postionBuilder;
        this.componentFactoryResolver = componentFactoryResolver;
        this.appRef = appRef;
        this.injector = injector;
        this.ngZone = ngZone;
    }
    /**
     * @param {?} overlayConfig
     * @return {?}
     */
    create(overlayConfig) {
        return this.createOverlayRef(this.createPaneElement(), overlayConfig);
    }
    /**
     * @return {?}
     */
    position() {
        return this.postionBuilder;
    }
    /**
     * @return {?}
     */
    createPaneElement() {
        let /** @type {?} */ pane = document.createElement('div');
        pane.id = `cui-overlay-${nextUniqueId++}`;
        pane.classList.add('cui-overlay-pane');
        this.overlayContainer.getContainerElement().appendChild(pane);
        return pane;
    }
    /**
     * @param {?} pane
     * @return {?}
     */
    createPortalHost(pane) {
        return new DomPortalHost(pane, this.componentFactoryResolver, this.appRef, this.injector);
    }
    /**
     * @param {?} pane
     * @param {?} overlayConfig
     * @return {?}
     */
    createOverlayRef(pane, overlayConfig) {
        let /** @type {?} */ portalHost = this.createPortalHost(pane);
        return new OverlayRef(portalHost, pane, this.ngZone, overlayConfig);
    }
}
Overlay.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
Overlay.ctorParameters = () => [
    { type: OverlayContainer, },
    { type: OverlayPositionBuilder, },
    { type: ComponentFactoryResolver, },
    { type: ApplicationRef, },
    { type: Injector, },
    { type: NgZone, },
];
function Overlay_tsickle_Closure_declarations() {
    /** @type {?} */
    Overlay.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    Overlay.ctorParameters;
    /** @type {?} */
    Overlay.prototype.overlayContainer;
    /** @type {?} */
    Overlay.prototype.postionBuilder;
    /** @type {?} */
    Overlay.prototype.componentFactoryResolver;
    /** @type {?} */
    Overlay.prototype.appRef;
    /** @type {?} */
    Overlay.prototype.injector;
    /** @type {?} */
    Overlay.prototype.ngZone;
}
export const /** @type {?} */ OVERLAY_PROVIDERS = [
    Overlay,
    OverlayPositionBuilder,
    OVERLAY_CONTAINER_PROVIDER
];
