import { Component, ViewEncapsulation, Input, ViewChild, Output, EventEmitter } from '@angular/core';
import * as Cropper from 'cropperjs';
export class ImageCropperComponent {
    constructor() {
        this.viewMode = 2;
        this.moveable = true;
        this.scalable = true;
        this.zoomable = true;
        this.export = new EventEmitter();
        this.ready = new EventEmitter();
        this.origin = new EventEmitter(); // Cropper object
        this.isLoading = true;
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    imageLoaded(ev) {
        this.loadError = false;
        let /** @type {?} */ image = (ev.target);
        this.imageElement = image;
        image.crossOrigin = 'anonymous';
        if (this.cropper) {
            this.cropper.destroy();
        }
        // debugger;
        image.addEventListener('ready', () => {
            this.ready.emit(true);
            this.isLoading = false;
            if (this.cropbox) {
                this.cropper.setCropBoxData(this.cropbox);
            }
        });
        this.cropper = new Cropper(image, this.cropperOption());
        this.origin.emit(this.cropper);
    }
    /**
     * @param {?} files
     * @return {?}
     */
    read(files) {
        return new Promise((resolve, reject) => {
            if (!files || files.length === 0) {
                resolve();
                return;
            }
            const /** @type {?} */ file = files[0];
            if (/^image\/\w+$/.test(file.type)) {
                const /** @type {?} */ reader = new FileReader();
                reader.onerror = reject;
                reader.onabort = reject;
                reader.onload = (e) => {
                    console.log(e);
                    this.imageUrl = reader.result;
                    // this.isLoading = true;
                    this.loadError = false;
                };
                reader.readAsDataURL(file);
            }
            else {
                reject('Please choose an image file.');
            }
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    loadImageFile(event) {
        this.loadError = false;
        this.read(event.target.files).then(() => {
            event.target.value = '';
        }).catch((e) => {
            event.target.value = '';
            this.alert(e);
        });
    }
    /**
     * @param {?} e
     * @return {?}
     */
    dragover(e) {
        e.preventDefault();
    }
    /**
     * @param {?} e
     * @return {?}
     */
    drop(e) {
        e.preventDefault();
        this.read(e.dataTransfer.files).catch(this.alert);
    }
    /**
     * @param {?} e
     * @return {?}
     */
    alert(e) {
        window.alert(e && e.message ? e.message : e);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    imageLoadError(event) {
        this.loadError = true;
        this.isLoading = false;
    }
    /**
     * @param {?=} base64
     * @return {?}
     */
    exportCanvas(base64) {
        let /** @type {?} */ imageData = this.cropper.getImageData();
        let /** @type {?} */ cropData = this.cropper.getCropBoxData();
        let /** @type {?} */ canvas = this.cropper.getCroppedCanvas();
        let /** @type {?} */ data = {
            imageData,
            cropData,
        };
        let /** @type {?} */ promise = new Promise(resolve => {
            if (base64) {
                return resolve({
                    dataUrl: canvas.toDataURL('image/png'),
                });
            }
            // canvas.toBlob(blob => resolve({ blob }));
        });
        promise.then(res => {
            this.export.emit(Object.assign(data, res));
        });
    }
    /**
     * @return {?}
     */
    cropperOption() {
        let /** @type {?} */ aspectRatio = NaN;
        if (this.settings) {
            let { width, height } = this.settings;
            aspectRatio = width / height;
        }
        return Object.assign({
            aspectRatio,
            movable: this.moveable,
            scalable: this.scalable,
            zoomable: this.zoomable,
            viewMode: this.viewMode,
        }, this.cropperOptions);
    }
}
ImageCropperComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-cropper',
                templateUrl: './image-cropper.component.html',
                styleUrls: ['./image-cropper.component.scss', './cropper.scss'],
                encapsulation: ViewEncapsulation.None,
            },] },
];
/**
 * @nocollapse
 */
ImageCropperComponent.ctorParameters = () => [];
ImageCropperComponent.propDecorators = {
    'imageUrl': [{ type: Input },],
    'viewMode': [{ type: Input },],
    'moveable': [{ type: Input },],
    'scalable': [{ type: Input },],
    'zoomable': [{ type: Input },],
    'settings': [{ type: Input },],
    'cropbox': [{ type: Input },],
    'cropperOptions': [{ type: Input },],
    'image': [{ type: ViewChild, args: ['image',] },],
    'export': [{ type: Output },],
    'ready': [{ type: Output },],
    'origin': [{ type: Output },],
    'contentTpl': [{ type: Input },],
};
function ImageCropperComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    ImageCropperComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ImageCropperComponent.ctorParameters;
    /** @type {?} */
    ImageCropperComponent.propDecorators;
    /** @type {?} */
    ImageCropperComponent.prototype.imageUrl;
    /** @type {?} */
    ImageCropperComponent.prototype.viewMode;
    /** @type {?} */
    ImageCropperComponent.prototype.moveable;
    /** @type {?} */
    ImageCropperComponent.prototype.scalable;
    /** @type {?} */
    ImageCropperComponent.prototype.zoomable;
    /** @type {?} */
    ImageCropperComponent.prototype.settings;
    /** @type {?} */
    ImageCropperComponent.prototype.cropbox;
    /** @type {?} */
    ImageCropperComponent.prototype.cropperOptions;
    /** @type {?} */
    ImageCropperComponent.prototype.image;
    /** @type {?} */
    ImageCropperComponent.prototype.export;
    /** @type {?} */
    ImageCropperComponent.prototype.ready;
    /** @type {?} */
    ImageCropperComponent.prototype.origin;
    /** @type {?} */
    ImageCropperComponent.prototype.contentTpl;
    /** @type {?} */
    ImageCropperComponent.prototype.isLoading;
    /** @type {?} */
    ImageCropperComponent.prototype.loadError;
    /** @type {?} */
    ImageCropperComponent.prototype.cropper;
    /** @type {?} */
    ImageCropperComponent.prototype.imageElement;
}
