export class TreeNode {
    /**
     * @param {?} data
     * @param {?} parent
     * @param {?} treeModel
     * @param {?} index
     */
    constructor(data, parent, treeModel, index) {
        this.data = data;
        this.parent = parent;
        this.treeModel = treeModel;
        this.loading = false;
        this.index = index;
        if (this.getField('selected') || treeModel.isSelected(this)) {
            this.treeModel.addSelection(this);
        }
        if (this.getField('children')) {
            this.initChildren();
        }
        else {
            if (this.getField('hasChildren') && this.getField('expanded')
                && this.treeModel.config.async && this.treeModel.config.async.enable) {
                this.loadChildren();
            }
        }
    }
    /**
     * @return {?}
     */
    get config() {
        return this.treeModel.config;
    }
    /**
     * @return {?}
     */
    get id() {
        return this.getField('id');
    }
    /**
     * @return {?}
     */
    get label() {
        return this.getField('label');
    }
    /**
     * @return {?}
     */
    get selectable() {
        return this.getField('selectable');
    }
    /**
     * @return {?}
     */
    get hasChildren() {
        return this.getField('hasChildren') || !!this.children;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hasChildren(val) {
        this.setField('hasChildren', val);
    }
    /**
     * @return {?}
     */
    get leaf() {
        return !this.hasChildren;
    }
    /**
     * @return {?}
     */
    get expanded() {
        return this.getField('expanded');
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set expanded(val) {
        this.setField('expanded', val);
    }
    /**
     * @return {?}
     */
    get icon() {
        return this.getField('icon');
    }
    /**
     * @return {?}
     */
    get expandedIcon() {
        return this.getField('expandedIcon');
    }
    /**
     * @return {?}
     */
    get collapsedIcon() {
        return this.getField('collapsedIcon');
    }
    /**
     * @return {?}
     */
    get partialSelected() {
        return this.getField('partialSelected');
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set partialSelected(val) {
        this.setField('partialSelected', val);
    }
    /**
     * @return {?}
     */
    get selected() {
        return this.getField('selected');
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set selected(val) {
        if (val) {
            this.treeModel.addSelection(this);
        }
        else {
            this.treeModel.removeSelection(this);
        }
        this.setField('selected', val);
    }
    /**
     * @return {?}
     */
    get children() {
        this.mergeChildren();
        return this._children;
    }
    /**
     * @return {?}
     */
    get disabled() {
        return this.getField('disabled');
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set disabled(val) {
        this.setField('disabled', val);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getField(key) {
        return this.data[this.config.data.key[key] || key];
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    setField(key, value) {
        this.data[this.config.data.key[key] || key] = value;
    }
    /**
     * @param {?=} force
     * @return {?}
     */
    loadChildren(force = false) {
        let /** @type {?} */ dataChildren = this.getField('children');
        if (!dataChildren || force) {
            if (this.config.async && this.config.async.enable && this.hasChildren && !this.loading) {
                this.loading = true;
                this.config.async.loadChildren(this).subscribe(data => {
                    if (this.config.async.dataFilter) {
                        data = this.config.async.dataFilter(data, this);
                    }
                    this.setField('children', data);
                    this.mergeChildren();
                    this.loading = false;
                });
            }
        }
    }
    /**
     * @return {?}
     */
    initChildren() {
        this._children = this.getField('children') ?
            this.getField('children').map((child, idx) => new TreeNode(child, this, this.treeModel, idx)) : [];
    }
    /**
     * @return {?}
     */
    mergeChildren() {
        let /** @type {?} */ dataChildren = this.getField('children');
        if (dataChildren) {
            // TODO: 优化比较children
            if (!this._children) {
                this.initChildren();
            }
            else {
                let /** @type {?} */ childrenOld = this._children;
                let /** @type {?} */ childrenNew = [];
                dataChildren.forEach((dataChild, i) => {
                    let /** @type {?} */ co = childrenOld.find(it => it.data == dataChild);
                    if (!co) {
                        co = new TreeNode(dataChild, this, this.treeModel, i);
                    }
                    childrenNew.push(co);
                });
                this._children = childrenNew;
            }
        }
        else {
            if (this._children) {
                this._children = null;
            }
        }
    }
}
function TreeNode_tsickle_Closure_declarations() {
    /** @type {?} */
    TreeNode.prototype.index;
    /** @type {?} */
    TreeNode.prototype._children;
    /** @type {?} */
    TreeNode.prototype.loading;
    /** @type {?} */
    TreeNode.prototype.data;
    /** @type {?} */
    TreeNode.prototype.parent;
    /** @type {?} */
    TreeNode.prototype.treeModel;
}
