import { Component, Input, Output, EventEmitter, ContentChildren, ContentChild } from '@angular/core';
import { CuiTemplateDirective } from '../../../core/template/template.directive';
import { TreeModel } from '../model/tree-model';
export class TreeComponent {
    /**
     * @param {?} treeModel
     */
    constructor(treeModel) {
        this.treeModel = treeModel;
        // @Input() config: CuiTreeConfig;
        this.metaKeySelection = true;
        this.propagateSelectionUp = true;
        this.propagateSelectionDown = true;
        this.selectionChange = new EventEmitter();
        this.nodeSelect = new EventEmitter();
        this.nodeUnselect = new EventEmitter();
        this.nodeExpand = new EventEmitter();
        this.nodeCollapse = new EventEmitter();
        // this.selectionDiffer = differ.find([]).create(null);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) { }
    /**
     * @param {?} config
     * @return {?}
     */
    set config(config) { }
    /**
     * @return {?}
     */
    get config() {
        return this.treeModel.config;
    }
    /**
     * @return {?}
     */
    get selection() {
        return this.treeModel.selection;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set selection(val) {
        this.treeModel.selection = val;
    }
    /**
     * @return {?}
     */
    get isSingleSelectionMode() {
        return this.selectionMode && this.selectionMode == 'single';
    }
    /**
     * @return {?}
     */
    get isMultipleSelectionMode() {
        return this.selectionMode && this.selectionMode == 'multiple';
    }
    /**
     * @return {?}
     */
    get isCheckboxSelectionMode() {
        return this.selectionMode && this.selectionMode == 'checkbox';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // console.log(this.value);
        // this.config = Object.assign({}, defaultTreeConfig, this.config);
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        if (this.templates.length) {
            // cache template
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.treeModel.setData({
            nodes: changes.value && changes.value.currentValue,
            config: changes.config && changes.config.currentValue,
            events: {
                'selectionChange': this.selectionChange,
                'nodeSelect': this.nodeSelect,
                'nodeUnselect': this.nodeUnselect,
                'nodeExpand': this.nodeExpand,
                'nodeCollapse': this.nodeCollapse,
            }
        });
        // console.log('changes', changes)
        // console.log('treeModel', this.treeModel);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
    }
    /**
     * @param {?} event
     * @param {?} node
     * @return {?}
     */
    onNodeClick(event, node) {
        let /** @type {?} */ eventTarget = (event.target);
        if (eventTarget.className && eventTarget.className.indexOf('cui-treenode-toggler') >= 0) {
            return;
        }
        if (!this.selectionMode || node.selectable === false) {
            return;
        }
        let /** @type {?} */ index = this.findIndexInSelection(node);
        let /** @type {?} */ selected = index >= 0;
        if (this.isCheckboxSelectionMode) {
            if (selected) {
                // 事件/属性传播属性
                if (this.propagateSelectionDown) {
                    this.propagateDown(node, false);
                }
                else {
                    this.selection = this.selection.filter((n, i) => i != index);
                }
                if (this.propagateSelectionUp && node.parent) {
                    this.propagateUp(node.parent, false);
                }
                this.selectionChange.emit(this.selection);
                this.nodeUnselect.emit({ originalEvent: event, node: node });
            }
            else {
                // 事件/属性传播属性
                if (this.propagateSelectionDown) {
                    this.propagateDown(node, true);
                }
                else {
                    this.selection = [...this.selection || [], node];
                }
                if (this.propagateSelectionUp && node.parent) {
                    this.propagateUp(node.parent, true);
                }
                this.selectionChange.emit(this.selection);
                this.nodeSelect.emit({ originalEvent: event, node: node });
            }
            // this.selectionChange.emit(this.selection);
            // this.nodeSelect.emit({ originalEvent: event, node: node });
            return;
        }
        let /** @type {?} */ metaSelection = this.nodeTouched ? false : this.metaKeySelection;
        if (metaSelection) {
            let /** @type {?} */ metaKey = (event.metaKey || event.ctrlKey);
            if (selected && metaKey) {
                if (this.isSingleSelectionMode) {
                    this.selectionChange.emit(null);
                }
                else {
                    this.selection = this.selection.filter((val, i) => i != index);
                    this.selectionChange.emit(this.selection);
                }
                this.nodeUnselect.emit({ originalEvent: event, node: node });
            }
            else {
                if (this.isSingleSelectionMode) {
                    this.selection = [node];
                    this.selectionChange.emit([node]);
                }
                if (this.isMultipleSelectionMode) {
                    this.selection = (!metaKey) ? [] : this.selection || [];
                    this.selection = [...this.selection, node];
                    this.selectionChange.emit(this.selection);
                }
                this.nodeSelect.emit({ originalEvent: event, node: node });
            }
        }
        else {
            if (this.isSingleSelectionMode) {
                if (selected) {
                    this.selection = null;
                    this.nodeUnselect.emit({ originalEvent: event, node: node });
                }
                else {
                    this.selection = [node];
                    this.nodeSelect.emit({ originalEvent: event, node: node });
                }
            }
            else {
                if (selected) {
                    this.selection = this.selection.filter((val, i) => i != index);
                    this.nodeUnselect.emit({ originalEvent: event, node: node });
                }
                else {
                    this.selection = [...this.selection || [], node];
                    this.nodeSelect.emit({ originalEvent: event, node: node });
                }
            }
            this.selectionChange.emit(this.selection);
        }
        this.nodeTouched = false;
    }
    /**
     * @return {?}
     */
    onNodeTouchEnd() {
        this.nodeTouched = true;
    }
    /**
     * @param {?} event
     * @param {?} node
     * @return {?}
     */
    onNodeRightClick(event, node) {
        // TODO: 加载上下文菜单
    }
    /**
     * @param {?} node
     * @return {?}
     */
    findIndexInSelection(node) {
        let /** @type {?} */ index = -1;
        if (this.selectionMode && this.selection) {
            index = this.selection.findIndex((n, i) => n == node || n.id == node.id);
        }
        return index;
    }
    /**
     * @param {?} node
     * @param {?} select
     * @return {?}
     */
    propagateUp(node, select) {
        if (node.children && node.children.length) {
            let /** @type {?} */ selectedCount = 0;
            let /** @type {?} */ childPartialSelected = false;
            for (let /** @type {?} */ child of node.children) {
                if (this.isSelected(child)) {
                    selectedCount++;
                }
                else if (child.partialSelected) {
                    childPartialSelected = true;
                }
            }
            if (select && selectedCount == node.children.length) {
                this.selection = [...this.selection || [], node];
                node.partialSelected = false;
            }
            else {
                if (!select) {
                    let /** @type {?} */ index = this.findIndexInSelection(node);
                    if (index >= 0) {
                        this.selection = this.selection.filter((val, i) => i != index);
                    }
                }
                if (childPartialSelected || selectedCount > 0 && selectedCount != node.children.length) {
                    node.partialSelected = true;
                }
                else {
                    node.partialSelected = false;
                }
            }
        }
        let /** @type {?} */ parent = node.parent;
        if (parent) {
            this.propagateUp(parent, select);
        }
    }
    /**
     * @param {?} node
     * @param {?} select
     * @return {?}
     */
    propagateDown(node, select) {
        let /** @type {?} */ index = this.findIndexInSelection(node);
        if (select && index == -1) {
            this.selection = [...this.selection || [], node];
        }
        else if (!select && index > -1) {
            this.selection = this.selection.filter((val, i) => i != index);
        }
        node.partialSelected = false;
        if (node.children && node.children.length) {
            for (let /** @type {?} */ child of node.children) {
                this.propagateDown(child, select);
            }
        }
    }
    /**
     * @param {?} node
     * @return {?}
     */
    isSelected(node) {
        return this.findIndexInSelection(node) != -1;
    }
    /**
     * @return {?}
     */
    getTemplateForNode() {
    }
    /**
     * @param {?} key
     * @param {?} value
     * @param {?=} parentNode
     * @return {?}
     */
    getNodeByParam(key, value, parentNode) {
        let /** @type {?} */ results = this.treeModel.filter(node => {
            return node[key] == value || node.data[key] == value;
        }, parentNode);
        if (results && results.length > 0) {
            return results[0];
        }
        return null;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    getNodeById(id) {
        return this.getNodeByParam('id', id);
    }
    /**
     * @param {?} node
     * @return {?}
     */
    selectNode(node) {
        this.treeModel.addSelection(node);
    }
    /**
     * @param {?} node
     * @return {?}
     */
    removeSelection(node) {
        this.treeModel.removeSelection(node);
    }
    /**
     * @param {?=} node
     * @param {?=} mode
     * @return {?}
     */
    refresh(node, mode = 'rate') {
        switch (mode) {
            case 'children':
                this.refreshChildren(node);
                break;
            case 'rate':
            default:
                this.refreshRate(node);
                break;
        }
    }
    /**
     * @param {?} node
     * @return {?}
     */
    refreshRate(node) {
        let /** @type {?} */ parent = node && node.parent;
        this.refreshChildren(parent);
    }
    /**
     * @param {?} node
     * @return {?}
     */
    refreshChildren(node) {
        node = node || this.treeModel.virtualRoot;
        if (node) {
            if (!node.hasChildren) {
                node.hasChildren = true;
                node.expanded = true;
            }
            node.loadChildren(true);
        }
    }
}
TreeComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tree',
                templateUrl: './tree.component.html',
                styleUrls: ['./tree.component.scss'],
                providers: [TreeModel]
            },] },
];
/**
 * @nocollapse
 */
TreeComponent.ctorParameters = () => [
    { type: TreeModel, },
];
TreeComponent.propDecorators = {
    'id': [{ type: Input },],
    'selectionMode': [{ type: Input },],
    'style': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'metaKeySelection': [{ type: Input },],
    'propagateSelectionUp': [{ type: Input },],
    'propagateSelectionDown': [{ type: Input },],
    'selectionChange': [{ type: Output },],
    'nodeSelect': [{ type: Output },],
    'nodeUnselect': [{ type: Output },],
    'nodeExpand': [{ type: Output },],
    'nodeCollapse': [{ type: Output },],
    'templates': [{ type: ContentChildren, args: [CuiTemplateDirective,] },],
    'nodeTemplate': [{ type: ContentChild, args: ["nodeTemplate",] },],
    'value': [{ type: Input },],
    'config': [{ type: Input },],
};
function TreeComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    TreeComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    TreeComponent.ctorParameters;
    /** @type {?} */
    TreeComponent.propDecorators;
    /** @type {?} */
    TreeComponent.prototype.id;
    /** @type {?} */
    TreeComponent.prototype.selectionMode;
    /** @type {?} */
    TreeComponent.prototype.style;
    /** @type {?} */
    TreeComponent.prototype.styleClass;
    /** @type {?} */
    TreeComponent.prototype.metaKeySelection;
    /** @type {?} */
    TreeComponent.prototype.propagateSelectionUp;
    /** @type {?} */
    TreeComponent.prototype.propagateSelectionDown;
    /** @type {?} */
    TreeComponent.prototype.selectionChange;
    /** @type {?} */
    TreeComponent.prototype.nodeSelect;
    /** @type {?} */
    TreeComponent.prototype.nodeUnselect;
    /** @type {?} */
    TreeComponent.prototype.nodeExpand;
    /** @type {?} */
    TreeComponent.prototype.nodeCollapse;
    /** @type {?} */
    TreeComponent.prototype.templates;
    /** @type {?} */
    TreeComponent.prototype.nodeTemplate;
    /** @type {?} */
    TreeComponent.prototype.nodes;
    /** @type {?} */
    TreeComponent.prototype.nodeTouched;
    /** @type {?} */
    TreeComponent.prototype.treeModel;
}
