import { Component, ElementRef, Input, Output, EventEmitter, forwardRef, Renderer2, ViewChild, ChangeDetectorRef } from '@angular/core';
import { trigger, state, style, transition, animate } from '@angular/animations';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS } from '@angular/forms';
import { DomHandler } from '../../core/dom/dom-handler';
export const /** @type {?} */ CALENDAR_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => CalendarComponent),
    multi: true
};
export const /** @type {?} */ CALENDAR_VALIDATOR = {
    provide: NG_VALIDATORS,
    useExisting: forwardRef(() => CalendarComponent),
    multi: true
};
export class CalendarComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} renderer
     * @param {?} cd
     */
    constructor(el, domHandler, renderer, cd) {
        this.el = el;
        this.domHandler = domHandler;
        this.renderer = renderer;
        this.cd = cd;
        this.dateFormat = 'yy/mm/dd';
        this.inline = false;
        this.showOtherMonths = true;
        this.icon = 'fa-calendar';
        this.shortYearCutoff = '+10';
        this.hourFormat = '24';
        this.stepHour = 1;
        this.stepMinute = 1;
        this.stepSecond = 1;
        this.showSeconds = false;
        this.showOnFocus = true;
        this.dataType = 'date';
        this.onFocus = new EventEmitter();
        this.onBlur = new EventEmitter();
        this.onClose = new EventEmitter();
        this.onSelect = new EventEmitter();
        this.onInput = new EventEmitter();
        this._locale_en = {
            firstDayOfWeek: 0,
            dayNames: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
            dayNamesShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
            dayNamesMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"],
            monthNames: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
            monthNamesShort: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
        };
        this._locale = {
            firstDayOfWeek: 0,
            dayNames: ["星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六"],
            dayNamesShort: ["周日", "周一", "周二", "周三", "周四", "周五", "周六"],
            dayNamesMin: ["日", "一", "二", "三", "四", "五", "六"],
            monthNames: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
            monthNamesShort: ["1月", "2月", "3月", "4月", "5月", "6月", "7月", "8月", "9月", "10月", "11月", "12月"]
        };
        this.closeOverlay = true;
        this.inputFieldValue = null;
        this._isValid = true;
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
    }
    /**
     * @return {?}
     */
    get minDate() {
        return this._minDate;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    set minDate(date) {
        this._minDate = date;
        this.createMonth(this.currentMonth, this.currentYear);
    }
    /**
     * @return {?}
     */
    get maxDate() {
        return this._maxDate;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    set maxDate(date) {
        this._maxDate = date;
        this.createMonth(this.currentMonth, this.currentYear);
    }
    /**
     * @return {?}
     */
    get locale() {
        return this._locale;
    }
    /**
     * @param {?} newLocale
     * @return {?}
     */
    set locale(newLocale) {
        this._locale = newLocale;
        this.createWeekDays();
        this.createMonth(this.currentMonth, this.currentYear);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        let /** @type {?} */ date = this.defaultDate || new Date();
        this.createWeekDays();
        this.currentMonth = date.getMonth();
        this.currentYear = date.getFullYear();
        this.pm = date.getHours() > 11;
        if (this.showTime) {
            this.currentMinute = date.getMinutes();
            this.currentSecond = date.getSeconds();
            if (this.hourFormat == '12') {
                this.currentHour = date.getHours() == 0 ? 12 : date.getHours() % 12;
            }
            else {
                this.currentHour = date.getHours();
            }
        }
        else if (this.timeOnly) {
            this.currentMinute = 0;
            this.currentHour = 0;
            this.currentSecond = 0;
        }
        this.createMonth(this.currentMonth, this.currentYear);
        this.ticksTo1970 = (((1970 - 1) * 365 + Math.floor(1970 / 4) - Math.floor(1970 / 100) +
            Math.floor(1970 / 400)) * 24 * 60 * 60 * 10000000);
        if (this.yearNavigator && this.yearRange) {
            this.yearOptions = [];
            let /** @type {?} */ years = this.yearRange.split(':'), /** @type {?} */ yearStart = parseInt(years[0]), /** @type {?} */ yearEnd = parseInt(years[1]);
            for (let /** @type {?} */ i = yearStart; i <= yearEnd; i++) {
                this.yearOptions.push(i);
            }
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (!this.inline && this.appendTo) {
            if (this.appendTo === 'body') {
                document.body.appendChild(this.overlayViewChild.nativeElement);
            }
            else {
                this.domHandler.appendChild(this.overlayViewChild.nativeElement, this.appendTo);
            }
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        if (this.overlayShown) {
            this.alignOverlay();
            this.overlayShown = false;
        }
    }
    /**
     * @return {?}
     */
    createWeekDays() {
        this.weekDays = [];
        let /** @type {?} */ dayIndex = this.locale.firstDayOfWeek;
        for (let /** @type {?} */ i = 0; i < 7; i++) {
            this.weekDays.push(this.locale.dayNamesMin[dayIndex]);
            dayIndex = (dayIndex == 6) ? 0 : ++dayIndex;
        }
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    createMonth(month, year) {
        this.dates = [];
        this.currentMonth = month;
        this.currentYear = year;
        this.currentMonthText = this.locale.monthNames[month];
        let /** @type {?} */ firstDay = this.getFirstDayOfMonthIndex(month, year);
        let /** @type {?} */ daysLength = this.getDaysCountInMonth(month, year);
        let /** @type {?} */ prevMonthDaysLength = this.getDaysCountInPrevMonth(month, year);
        let /** @type {?} */ sundayIndex = this.getSundayIndex();
        let /** @type {?} */ dayNo = 1;
        let /** @type {?} */ today = new Date();
        for (let /** @type {?} */ i = 0; i < 6; i++) {
            let /** @type {?} */ week = [];
            if (i == 0) {
                for (let /** @type {?} */ j = (prevMonthDaysLength - firstDay + 1); j <= prevMonthDaysLength; j++) {
                    let /** @type {?} */ prev = this.getPreviousMonthAndYear(month, year);
                    week.push({
                        day: j, month: prev.month, year: prev.year, otherMonth: true,
                        today: this.isToday(today, j, prev.month, prev.year), selectable: this.isSelectable(j, prev.month, prev.year)
                    });
                }
                let /** @type {?} */ remainingDaysLength = 7 - week.length;
                for (let /** @type {?} */ j = 0; j < remainingDaysLength; j++) {
                    week.push({
                        day: dayNo, month: month, year: year, today: this.isToday(today, dayNo, month, year),
                        selectable: this.isSelectable(dayNo, month, year)
                    });
                    dayNo++;
                }
            }
            else {
                for (let /** @type {?} */ j = 0; j < 7; j++) {
                    if (dayNo > daysLength) {
                        let /** @type {?} */ next = this.getNextMonthAndYear(month, year);
                        week.push({
                            day: dayNo - daysLength, month: next.month, year: next.year, otherMonth: true,
                            today: this.isToday(today, dayNo - daysLength, next.month, next.year),
                            selectable: this.isSelectable((dayNo - daysLength), next.month, next.year)
                        });
                    }
                    else {
                        week.push({
                            day: dayNo, month: month, year: year, today: this.isToday(today, dayNo, month, year),
                            selectable: this.isSelectable(dayNo, month, year)
                        });
                    }
                    dayNo++;
                }
            }
            this.dates.push(week);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    prevMonth(event) {
        if (this.disabled) {
            event.preventDefault();
            return;
        }
        if (this.currentMonth === 0) {
            this.currentMonth = 11;
            this.currentYear--;
        }
        else {
            this.currentMonth--;
        }
        this.createMonth(this.currentMonth, this.currentYear);
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    nextMonth(event) {
        if (this.disabled) {
            event.preventDefault();
            return;
        }
        if (this.currentMonth === 11) {
            this.currentMonth = 0;
            this.currentYear++;
        }
        else {
            this.currentMonth++;
        }
        this.createMonth(this.currentMonth, this.currentYear);
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @param {?} dateMeta
     * @return {?}
     */
    onDateSelect(event, dateMeta) {
        if (this.disabled || !dateMeta.selectable) {
            event.preventDefault();
            return;
        }
        if (dateMeta.otherMonth) {
            if (this.selectOtherMonths) {
                this.currentMonth = dateMeta.month;
                this.currentYear = dateMeta.year;
                this.createMonth(this.currentMonth, this.currentYear);
                this.selectDate(dateMeta);
            }
        }
        else {
            this.selectDate(dateMeta);
        }
        this.dateClick = true;
        this.updateInputfield();
        event.preventDefault();
    }
    /**
     * @return {?}
     */
    updateInputfield() {
        if (this.value) {
            let /** @type {?} */ formattedValue;
            if (this.timeOnly) {
                formattedValue = this.formatTime(this.value);
            }
            else {
                formattedValue = this.formatDate(this.value, this.dateFormat);
                if (this.showTime) {
                    formattedValue += ' ' + this.formatTime(this.value);
                }
            }
            this.inputFieldValue = formattedValue;
        }
        else {
            this.inputFieldValue = '';
        }
        this.updateFilledState();
    }
    /**
     * @param {?} dateMeta
     * @return {?}
     */
    selectDate(dateMeta) {
        if (this.utc) {
            this.value = new Date(Date.UTC(dateMeta.year, dateMeta.month, dateMeta.day));
        }
        else {
            this.value = new Date(dateMeta.year, dateMeta.month, dateMeta.day);
        }
        if (this.showTime) {
            if (this.hourFormat === '12' && this.pm && this.currentHour != 12) {
                this.value.setHours(this.currentHour + 12);
            }
            else {
                this.value.setHours(this.currentHour);
            }
            this.value.setMinutes(this.currentMinute);
            this.value.setSeconds(this.currentSecond);
        }
        this._isValid = true;
        this.updateModel();
        this.onSelect.emit(this.value);
    }
    /**
     * @return {?}
     */
    updateModel() {
        if (this.dataType == 'date') {
            this.onModelChange(this.value);
        }
        else if (this.dataType == 'string') {
            if (this.timeOnly) {
                this.onModelChange(this.formatTime(this.value));
            }
            else {
                this.onModelChange(this.formatDate(this.value, this.dateFormat));
            }
        }
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    getFirstDayOfMonthIndex(month, year) {
        let /** @type {?} */ day = new Date();
        day.setDate(1);
        day.setMonth(month);
        day.setFullYear(year);
        let /** @type {?} */ dayIndex = day.getDay() + this.getSundayIndex();
        return dayIndex >= 7 ? dayIndex - 7 : dayIndex;
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    getDaysCountInMonth(month, year) {
        return 32 - this.daylightSavingAdjust(new Date(year, month, 32)).getDate();
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    getDaysCountInPrevMonth(month, year) {
        let /** @type {?} */ prev = this.getPreviousMonthAndYear(month, year);
        return this.getDaysCountInMonth(prev.month, prev.year);
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    getPreviousMonthAndYear(month, year) {
        let /** @type {?} */ m, /** @type {?} */ y;
        if (month === 0) {
            m = 11;
            y = year - 1;
        }
        else {
            m = month - 1;
            y = year;
        }
        return { 'month': m, 'year': y };
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    getNextMonthAndYear(month, year) {
        let /** @type {?} */ m, /** @type {?} */ y;
        if (month === 11) {
            m = 0;
            y = year + 1;
        }
        else {
            m = month + 1;
            y = year;
        }
        return { 'month': m, 'year': y };
    }
    /**
     * @return {?}
     */
    getSundayIndex() {
        return this.locale.firstDayOfWeek > 0 ? 7 - this.locale.firstDayOfWeek : 0;
    }
    /**
     * @param {?} dateMeta
     * @return {?}
     */
    isSelected(dateMeta) {
        if (this.value) {
            return this.value.getDate() === dateMeta.day &&
                this.value.getMonth() === dateMeta.month && this.value.getFullYear() === dateMeta.year;
        }
        else {
            return false;
        }
    }
    /**
     * @param {?} today
     * @param {?} day
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    isToday(today, day, month, year) {
        return today.getDate() === day && today.getMonth() === month && today.getFullYear() === year;
    }
    /**
     * @param {?} day
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    isSelectable(day, month, year) {
        let /** @type {?} */ validMin = true;
        let /** @type {?} */ validMax = true;
        let /** @type {?} */ validDate = true;
        let /** @type {?} */ validDay = true;
        if (this.minDate) {
            if (this.minDate.getFullYear() > year) {
                validMin = false;
            }
            else if (this.minDate.getFullYear() === year) {
                if (this.minDate.getMonth() > month) {
                    validMin = false;
                }
                else if (this.minDate.getMonth() === month) {
                    if (this.minDate.getDate() > day) {
                        validMin = false;
                    }
                }
            }
        }
        if (this.maxDate) {
            if (this.maxDate.getFullYear() < year) {
                validMax = false;
            }
            else if (this.maxDate.getFullYear() === year) {
                if (this.maxDate.getMonth() < month) {
                    validMax = false;
                }
                else if (this.maxDate.getMonth() === month) {
                    if (this.maxDate.getDate() < day) {
                        validMax = false;
                    }
                }
            }
        }
        if (this.disabledDates) {
            validDate = !this.isDateDisabled(day, month, year);
        }
        if (this.disabledDays) {
            validDay = !this.isDayDisabled(day, month, year);
        }
        return validMin && validMax && validDate && validDay;
    }
    /**
     * @param {?} day
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    isDateDisabled(day, month, year) {
        if (this.disabledDates) {
            for (let /** @type {?} */ disabledDate of this.disabledDates) {
                if (disabledDate.getFullYear() === year && disabledDate.getMonth() === month && disabledDate.getDate() === day) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @param {?} day
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    isDayDisabled(day, month, year) {
        if (this.disabledDays) {
            let /** @type {?} */ weekday = new Date(year, month, day);
            let /** @type {?} */ weekdayNumber = weekday.getDay();
            return this.disabledDays.indexOf(weekdayNumber) !== -1;
        }
        return false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputFocus(event) {
        this.focus = true;
        if (this.showOnFocus) {
            this.showOverlay();
        }
        this.onFocus.emit(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputBlur(event) {
        this.focus = false;
        this.onBlur.emit(event);
        this.updateInputfield();
        this.onModelTouched();
    }
    /**
     * @param {?} event
     * @param {?} inputfield
     * @return {?}
     */
    onButtonClick(event, inputfield) {
        this.closeOverlay = false;
        if (!this.overlayViewChild.nativeElement.offsetParent || this.overlayViewChild.nativeElement.style.display === 'none') {
            inputfield.focus();
            this.showOverlay();
        }
        else {
            this.closeOverlay = true;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputKeydown(event) {
        this.isKeydown = true;
        if (event.keyCode === 9) {
            this.overlayVisible = false;
        }
    }
    /**
     * @param {?} m
     * @return {?}
     */
    onMonthDropdownChange(m) {
        this.currentMonth = parseInt(m);
        this.createMonth(this.currentMonth, this.currentYear);
    }
    /**
     * @param {?} y
     * @return {?}
     */
    onYearDropdownChange(y) {
        this.currentYear = parseInt(y);
        this.createMonth(this.currentMonth, this.currentYear);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    incrementHour(event) {
        let /** @type {?} */ newHour = this.currentHour + this.stepHour;
        if (this.hourFormat == '24') {
            this.currentHour = (newHour >= 24) ? (newHour - 24) : newHour;
        }
        else if (this.hourFormat == '12') {
            this.currentHour = (newHour >= 13) ? (newHour - 12) : newHour;
        }
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    decrementHour(event) {
        let /** @type {?} */ newHour = this.currentHour - this.stepHour;
        if (this.hourFormat == '24') {
            this.currentHour = (newHour < 0) ? (24 + newHour) : newHour;
        }
        else if (this.hourFormat == '12') {
            this.currentHour = (newHour <= 0) ? (12 + newHour) : newHour;
        }
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    incrementMinute(event) {
        let /** @type {?} */ newMinute = this.currentMinute + this.stepMinute;
        this.currentMinute = (newMinute > 59) ? newMinute - 60 : newMinute;
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    decrementMinute(event) {
        let /** @type {?} */ newMinute = this.currentMinute - this.stepMinute;
        this.currentMinute = (newMinute < 0) ? 60 + newMinute : newMinute;
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    incrementSecond(event) {
        let /** @type {?} */ newSecond = this.currentSecond + this.stepSecond;
        this.currentSecond = (newSecond > 59) ? newSecond - 60 : newSecond;
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    decrementSecond(event) {
        let /** @type {?} */ newSecond = this.currentSecond - this.stepSecond;
        this.currentSecond = (newSecond < 0) ? 60 + newSecond : newSecond;
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @return {?}
     */
    updateTime() {
        this.value = this.value || new Date();
        if (this.hourFormat === '12' && this.pm && this.currentHour != 12) {
            this.value.setHours(this.currentHour + 12);
        }
        else {
            this.value.setHours(this.currentHour);
        }
        this.value.setMinutes(this.currentMinute);
        this.value.setSeconds(this.currentSecond);
        this.updateModel();
        this.onSelect.emit(this.value);
        this.updateInputfield();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggleAMPM(event) {
        this.pm = !this.pm;
        this.updateTime();
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onUserInput(event) {
        // IE 11 Workaround for input placeholder : https://github.com/primefaces/primeng/issues/2026
        if (!this.isKeydown) {
            return;
        }
        this.isKeydown = false;
        let /** @type {?} */ val = event.target.value;
        try {
            this.value = this.parseValueFromString(val);
            this.updateUI();
            this._isValid = true;
        }
        catch (err) {
            // invalid date
            this.value = null;
            this._isValid = false;
        }
        this.filled = val != null && val.length;
        this.updateModel();
        this.onInput.emit(event);
    }
    /**
     * @param {?} text
     * @return {?}
     */
    parseValueFromString(text) {
        if (!text || text.trim().length === 0) {
            return null;
        }
        let /** @type {?} */ dateValue;
        let /** @type {?} */ parts = text.split(' ');
        if (this.timeOnly) {
            dateValue = new Date();
            this.populateTime(dateValue, parts[0], parts[1]);
        }
        else {
            if (this.showTime) {
                dateValue = this.parseDate(parts[0], this.dateFormat);
                this.populateTime(dateValue, parts[1], parts[2]);
            }
            else {
                dateValue = this.parseDate(text, this.dateFormat);
            }
        }
        return dateValue;
    }
    /**
     * @param {?} value
     * @param {?} timeString
     * @param {?} ampm
     * @return {?}
     */
    populateTime(value, timeString, ampm) {
        if (this.hourFormat == '12' && !ampm) {
            throw Error('Invalid Time');
        }
        this.pm = (ampm === 'PM' || ampm === 'pm');
        let /** @type {?} */ time = this.parseTime(timeString);
        value.setHours(time.hour);
        value.setMinutes(time.minute);
        value.setSeconds(time.second);
    }
    /**
     * @return {?}
     */
    updateUI() {
        let /** @type {?} */ val = this.value || this.defaultDate || new Date();
        this.createMonth(val.getMonth(), val.getFullYear());
        if (this.showTime || this.timeOnly) {
            let /** @type {?} */ hours = val.getHours();
            if (this.hourFormat == '12') {
                if (hours >= 12) {
                    this.currentHour = (hours == 12) ? 12 : hours - 12;
                }
                else {
                    this.currentHour = (hours == 0) ? 12 : hours;
                }
            }
            else {
                this.currentHour = val.getHours();
            }
            this.currentMinute = val.getMinutes();
            this.currentSecond = val.getSeconds();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDatePickerClick(event) {
        this.closeOverlay = this.dateClick;
    }
    /**
     * @return {?}
     */
    showOverlay() {
        this.overlayVisible = true;
        this.overlayShown = true;
        this.overlayViewChild.nativeElement.style.zIndex = String(++DomHandler.zindex);
        this.bindDocumentClickListener();
    }
    /**
     * @return {?}
     */
    alignOverlay() {
        if (this.appendTo) {
            this.domHandler.absolutePosition(this.overlayViewChild.nativeElement, this.inputfieldViewChild.nativeElement);
        }
        else {
            this.domHandler.relativePosition(this.overlayViewChild.nativeElement, this.inputfieldViewChild.nativeElement);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.value = value;
        if (this.value && typeof this.value === 'string') {
            this.value = this.parseValueFromString(this.value);
        }
        this.updateInputfield();
        this.updateUI();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setDisabledState(val) {
        this.disabled = val;
    }
    /**
     * @param {?} date
     * @param {?} format
     * @return {?}
     */
    formatDate(date, format) {
        if (!date) {
            return "";
        }
        let /** @type {?} */ iFormat, /** @type {?} */ lookAhead = (match) => {
            let /** @type {?} */ matches = (iFormat + 1 < format.length && format.charAt(iFormat + 1) === match);
            if (matches) {
                iFormat++;
            }
            return matches;
        }, /** @type {?} */ formatNumber = (match, value, len) => {
            let /** @type {?} */ num = "" + value;
            if (lookAhead(match)) {
                while (num.length < len) {
                    num = "0" + num;
                }
            }
            return num;
        }, /** @type {?} */ formatName = (match, value, shortNames, longNames) => {
            return (lookAhead(match) ? longNames[value] : shortNames[value]);
        }, /** @type {?} */ output = "", /** @type {?} */ literal = false;
        if (date) {
            for (iFormat = 0; iFormat < format.length; iFormat++) {
                if (literal) {
                    if (format.charAt(iFormat) === "'" && !lookAhead("'")) {
                        literal = false;
                    }
                    else {
                        output += format.charAt(iFormat);
                    }
                }
                else {
                    switch (format.charAt(iFormat)) {
                        case "d":
                            output += formatNumber("d", date.getDate(), 2);
                            break;
                        case "D":
                            output += formatName("D", date.getDay(), this.locale.dayNamesShort, this.locale.dayNames);
                            break;
                        case "o":
                            output += formatNumber("o", Math.round((new Date(date.getFullYear(), date.getMonth(), date.getDate()).getTime() - new Date(date.getFullYear(), 0, 0).getTime()) / 86400000), 3);
                            break;
                        case "m":
                            output += formatNumber("m", date.getMonth() + 1, 2);
                            break;
                        case "M":
                            output += formatName("M", date.getMonth(), this.locale.monthNamesShort, this.locale.monthNames);
                            break;
                        case "y":
                            output += (lookAhead("y") ? date.getFullYear() :
                                (date.getFullYear() % 100 < 10 ? "0" : "") + date.getFullYear() % 100);
                            break;
                        case "@":
                            output += date.getTime();
                            break;
                        case "!":
                            output += date.getTime() * 10000 + this.ticksTo1970;
                            break;
                        case "'":
                            if (lookAhead("'")) {
                                output += "'";
                            }
                            else {
                                literal = true;
                            }
                            break;
                        default:
                            output += format.charAt(iFormat);
                    }
                }
            }
        }
        return output;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    formatTime(date) {
        if (!date) {
            return '';
        }
        let /** @type {?} */ output = '';
        let /** @type {?} */ hours = date.getHours();
        let /** @type {?} */ minutes = date.getMinutes();
        let /** @type {?} */ seconds = date.getSeconds();
        if (this.hourFormat == '12' && hours > 11 && hours != 12) {
            hours -= 12;
        }
        output += (hours < 10) ? '0' + hours : hours;
        output += ':';
        output += (minutes < 10) ? '0' + minutes : minutes;
        if (this.showSeconds) {
            output += ':';
            output += (seconds < 10) ? '0' + seconds : seconds;
        }
        if (this.hourFormat == '12') {
            output += date.getHours() > 11 ? ' PM' : ' AM';
        }
        return output;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    parseTime(value) {
        let /** @type {?} */ tokens = value.split(':');
        let /** @type {?} */ validTokenLength = this.showSeconds ? 3 : 2;
        if (tokens.length !== validTokenLength) {
            throw Error("Invalid time");
        }
        let /** @type {?} */ h = parseInt(tokens[0]);
        let /** @type {?} */ m = parseInt(tokens[1]);
        let /** @type {?} */ s = this.showSeconds ? parseInt(tokens[2]) : null;
        if (isNaN(h) || isNaN(m) || h > 23 || m > 59 || (this.hourFormat == '12' && h > 12) || (this.showSeconds && (isNaN(s) || s > 59))) {
            throw Error("Invalid time");
        }
        else {
            if (this.hourFormat == '12' && h !== 12 && this.pm) {
                h += 12;
            }
            return { hour: h, minute: m, second: s };
        }
    }
    /**
     * @param {?} value
     * @param {?} format
     * @return {?}
     */
    parseDate(value, format) {
        if (format == null || value == null) {
            throw Error("Invalid arguments");
        }
        value = (typeof value === "object" ? value.toString() : value + "");
        if (value === "") {
            return null;
        }
        let /** @type {?} */ iFormat, /** @type {?} */ dim, /** @type {?} */ extra, /** @type {?} */ iValue = 0, /** @type {?} */ shortYearCutoff = (typeof this.shortYearCutoff !== "string" ?
            this.shortYearCutoff : new Date().getFullYear() % 100 + parseInt(this.shortYearCutoff, 10)), /** @type {?} */ year = -1, /** @type {?} */ month = -1, /** @type {?} */ day = -1, /** @type {?} */ doy = -1, /** @type {?} */ literal = false, /** @type {?} */ date, /** @type {?} */ lookAhead = (match) => {
            let /** @type {?} */ matches = (iFormat + 1 < format.length && format.charAt(iFormat + 1) === match);
            if (matches) {
                iFormat++;
            }
            return matches;
        }, /** @type {?} */ getNumber = (match) => {
            let /** @type {?} */ isDoubled = lookAhead(match), /** @type {?} */ size = (match === "@" ? 14 : (match === "!" ? 20 :
                (match === "y" && isDoubled ? 4 : (match === "o" ? 3 : 2)))), /** @type {?} */ minSize = (match === "y" ? size : 1), /** @type {?} */ digits = new RegExp("^\\d{" + minSize + "," + size + "}"), /** @type {?} */ num = value.substring(iValue).match(digits);
            if (!num) {
                throw Error("Missing number at position " + iValue);
            }
            iValue += num[0].length;
            return parseInt(num[0], 10);
        }, /** @type {?} */ getName = (match, shortNames, longNames) => {
            let /** @type {?} */ index = -1;
            let /** @type {?} */ arr = lookAhead(match) ? longNames : shortNames;
            let /** @type {?} */ names = [];
            for (let /** @type {?} */ i = 0; i < arr.length; i++) {
                names.push([i, arr[i]]);
            }
            names.sort((a, b) => {
                return -(a[1].length - b[1].length);
            });
            for (let /** @type {?} */ i = 0; i < names.length; i++) {
                let /** @type {?} */ name = names[i][1];
                if (value.substr(iValue, name.length).toLowerCase() === name.toLowerCase()) {
                    index = names[i][0];
                    iValue += name.length;
                    break;
                }
            }
            if (index !== -1) {
                return index + 1;
            }
            else {
                throw Error("Unknown name at position " + iValue);
            }
        }, /** @type {?} */ checkLiteral = () => {
            if (value.charAt(iValue) !== format.charAt(iFormat)) {
                throw Error("Unexpected literal at position " + iValue);
            }
            iValue++;
        };
        for (iFormat = 0; iFormat < format.length; iFormat++) {
            if (literal) {
                if (format.charAt(iFormat) === "'" && !lookAhead("'")) {
                    literal = false;
                }
                else {
                    checkLiteral();
                }
            }
            else {
                switch (format.charAt(iFormat)) {
                    case "d":
                        day = getNumber("d");
                        break;
                    case "D":
                        getName("D", this.locale.dayNamesShort, this.locale.dayNames);
                        break;
                    case "o":
                        doy = getNumber("o");
                        break;
                    case "m":
                        month = getNumber("m");
                        break;
                    case "M":
                        month = getName("M", this.locale.monthNamesShort, this.locale.monthNames);
                        break;
                    case "y":
                        year = getNumber("y");
                        break;
                    case "@":
                        date = new Date(getNumber("@"));
                        year = date.getFullYear();
                        month = date.getMonth() + 1;
                        day = date.getDate();
                        break;
                    case "!":
                        date = new Date((getNumber("!") - this.ticksTo1970) / 10000);
                        year = date.getFullYear();
                        month = date.getMonth() + 1;
                        day = date.getDate();
                        break;
                    case "'":
                        if (lookAhead("'")) {
                            checkLiteral();
                        }
                        else {
                            literal = true;
                        }
                        break;
                    default:
                        checkLiteral();
                }
            }
        }
        if (iValue < value.length) {
            extra = value.substr(iValue);
            if (!/^\s+/.test(extra)) {
                throw Error("Extra/unparsed characters found in date: " + extra);
            }
        }
        if (year === -1) {
            year = new Date().getFullYear();
        }
        else if (year < 100) {
            year += new Date().getFullYear() - new Date().getFullYear() % 100 +
                (year <= shortYearCutoff ? 0 : -100);
        }
        if (doy > -1) {
            month = 1;
            day = doy;
            do {
                dim = this.getDaysCountInMonth(year, month - 1);
                if (day <= dim) {
                    break;
                }
                month++;
                day -= dim;
            } while (true);
        }
        date = this.daylightSavingAdjust(new Date(year, month - 1, day));
        if (date.getFullYear() !== year || date.getMonth() + 1 !== month || date.getDate() !== day) {
            throw Error("Invalid date"); // E.g. 31/02/00
        }
        return date;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    daylightSavingAdjust(date) {
        if (!date) {
            return null;
        }
        date.setHours(date.getHours() > 12 ? date.getHours() + 2 : 0);
        return date;
    }
    /**
     * @return {?}
     */
    updateFilledState() {
        this.filled = this.inputFieldValue && this.inputFieldValue != '';
    }
    /**
     * @return {?}
     */
    bindDocumentClickListener() {
        if (!this.documentClickListener) {
            this.documentClickListener = this.renderer.listen('document', 'click', () => {
                if (this.closeOverlay) {
                    this.overlayVisible = false;
                    this.onClose.emit(event);
                }
                this.closeOverlay = true;
                this.dateClick = false;
                this.cd.detectChanges();
            });
        }
    }
    /**
     * @return {?}
     */
    unbindDocumentClickListener() {
        if (this.documentClickListener) {
            this.documentClickListener();
            this.documentClickListener = null;
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unbindDocumentClickListener();
        if (!this.inline && this.appendTo) {
            this.el.nativeElement.appendChild(this.overlayViewChild.nativeElement);
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        if (!this._isValid) {
            return { invalidDate: true };
        }
        return null;
    }
}
CalendarComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-calendar',
                templateUrl: './calendar.component.html',
                styleUrls: ['./calendar.component.scss'],
                animations: [
                    trigger('overlayState', [
                        state('hidden', style({
                            opacity: 0
                        })),
                        state('visible', style({
                            opacity: 1
                        })),
                        transition('visible => hidden', animate('400ms ease-in')),
                        transition('hidden => visible', animate('400ms ease-out'))
                    ])
                ],
                host: {
                    '[class.ui-inputwrapper-filled]': 'filled',
                    '[class.ui-inputwrapper-focus]': 'focus'
                },
                providers: [CALENDAR_VALUE_ACCESSOR, CALENDAR_VALIDATOR]
            },] },
];
/**
 * @nocollapse
 */
CalendarComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: Renderer2, },
    { type: ChangeDetectorRef, },
];
CalendarComponent.propDecorators = {
    'defaultDate': [{ type: Input },],
    'style': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'inputStyle': [{ type: Input },],
    'inputId': [{ type: Input },],
    'inputStyleClass': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'disabled': [{ type: Input },],
    'dateFormat': [{ type: Input },],
    'inline': [{ type: Input },],
    'showOtherMonths': [{ type: Input },],
    'selectOtherMonths': [{ type: Input },],
    'showIcon': [{ type: Input },],
    'icon': [{ type: Input },],
    'appendTo': [{ type: Input },],
    'readonlyInput': [{ type: Input },],
    'shortYearCutoff': [{ type: Input },],
    'monthNavigator': [{ type: Input },],
    'yearNavigator': [{ type: Input },],
    'yearRange': [{ type: Input },],
    'showTime': [{ type: Input },],
    'hourFormat': [{ type: Input },],
    'timeOnly': [{ type: Input },],
    'stepHour': [{ type: Input },],
    'stepMinute': [{ type: Input },],
    'stepSecond': [{ type: Input },],
    'showSeconds': [{ type: Input },],
    'required': [{ type: Input },],
    'showOnFocus': [{ type: Input },],
    'dataType': [{ type: Input },],
    'disabledDates': [{ type: Input },],
    'disabledDays': [{ type: Input },],
    'utc': [{ type: Input },],
    'onFocus': [{ type: Output },],
    'onBlur': [{ type: Output },],
    'onClose': [{ type: Output },],
    'onSelect': [{ type: Output },],
    'onInput': [{ type: Output },],
    'tabindex': [{ type: Input },],
    'overlayViewChild': [{ type: ViewChild, args: ['datepicker',] },],
    'inputfieldViewChild': [{ type: ViewChild, args: ['inputfield',] },],
    'minDate': [{ type: Input },],
    'maxDate': [{ type: Input },],
    'locale': [{ type: Input },],
};
function CalendarComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    CalendarComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    CalendarComponent.ctorParameters;
    /** @type {?} */
    CalendarComponent.propDecorators;
    /** @type {?} */
    CalendarComponent.prototype.defaultDate;
    /** @type {?} */
    CalendarComponent.prototype.style;
    /** @type {?} */
    CalendarComponent.prototype.styleClass;
    /** @type {?} */
    CalendarComponent.prototype.inputStyle;
    /** @type {?} */
    CalendarComponent.prototype.inputId;
    /** @type {?} */
    CalendarComponent.prototype.inputStyleClass;
    /** @type {?} */
    CalendarComponent.prototype.placeholder;
    /** @type {?} */
    CalendarComponent.prototype.disabled;
    /** @type {?} */
    CalendarComponent.prototype.dateFormat;
    /** @type {?} */
    CalendarComponent.prototype.inline;
    /** @type {?} */
    CalendarComponent.prototype.showOtherMonths;
    /** @type {?} */
    CalendarComponent.prototype.selectOtherMonths;
    /** @type {?} */
    CalendarComponent.prototype.showIcon;
    /** @type {?} */
    CalendarComponent.prototype.icon;
    /** @type {?} */
    CalendarComponent.prototype.appendTo;
    /** @type {?} */
    CalendarComponent.prototype.readonlyInput;
    /** @type {?} */
    CalendarComponent.prototype.shortYearCutoff;
    /** @type {?} */
    CalendarComponent.prototype.monthNavigator;
    /** @type {?} */
    CalendarComponent.prototype.yearNavigator;
    /** @type {?} */
    CalendarComponent.prototype.yearRange;
    /** @type {?} */
    CalendarComponent.prototype.showTime;
    /** @type {?} */
    CalendarComponent.prototype.hourFormat;
    /** @type {?} */
    CalendarComponent.prototype.timeOnly;
    /** @type {?} */
    CalendarComponent.prototype.stepHour;
    /** @type {?} */
    CalendarComponent.prototype.stepMinute;
    /** @type {?} */
    CalendarComponent.prototype.stepSecond;
    /** @type {?} */
    CalendarComponent.prototype.showSeconds;
    /** @type {?} */
    CalendarComponent.prototype.required;
    /** @type {?} */
    CalendarComponent.prototype.showOnFocus;
    /** @type {?} */
    CalendarComponent.prototype.dataType;
    /** @type {?} */
    CalendarComponent.prototype.disabledDates;
    /** @type {?} */
    CalendarComponent.prototype.disabledDays;
    /** @type {?} */
    CalendarComponent.prototype.utc;
    /** @type {?} */
    CalendarComponent.prototype.onFocus;
    /** @type {?} */
    CalendarComponent.prototype.onBlur;
    /** @type {?} */
    CalendarComponent.prototype.onClose;
    /** @type {?} */
    CalendarComponent.prototype.onSelect;
    /** @type {?} */
    CalendarComponent.prototype.onInput;
    /** @type {?} */
    CalendarComponent.prototype._locale_en;
    /** @type {?} */
    CalendarComponent.prototype._locale;
    /** @type {?} */
    CalendarComponent.prototype.tabindex;
    /** @type {?} */
    CalendarComponent.prototype.overlayViewChild;
    /** @type {?} */
    CalendarComponent.prototype.inputfieldViewChild;
    /** @type {?} */
    CalendarComponent.prototype.value;
    /** @type {?} */
    CalendarComponent.prototype.dates;
    /** @type {?} */
    CalendarComponent.prototype.weekDays;
    /** @type {?} */
    CalendarComponent.prototype.currentMonthText;
    /** @type {?} */
    CalendarComponent.prototype.currentMonth;
    /** @type {?} */
    CalendarComponent.prototype.currentYear;
    /** @type {?} */
    CalendarComponent.prototype.currentHour;
    /** @type {?} */
    CalendarComponent.prototype.currentMinute;
    /** @type {?} */
    CalendarComponent.prototype.currentSecond;
    /** @type {?} */
    CalendarComponent.prototype.pm;
    /** @type {?} */
    CalendarComponent.prototype.overlay;
    /** @type {?} */
    CalendarComponent.prototype.overlayVisible;
    /** @type {?} */
    CalendarComponent.prototype.overlayShown;
    /** @type {?} */
    CalendarComponent.prototype.closeOverlay;
    /** @type {?} */
    CalendarComponent.prototype.dateClick;
    /** @type {?} */
    CalendarComponent.prototype.calendarElement;
    /** @type {?} */
    CalendarComponent.prototype.documentClickListener;
    /** @type {?} */
    CalendarComponent.prototype.ticksTo1970;
    /** @type {?} */
    CalendarComponent.prototype.yearOptions;
    /** @type {?} */
    CalendarComponent.prototype.focus;
    /** @type {?} */
    CalendarComponent.prototype.isKeydown;
    /** @type {?} */
    CalendarComponent.prototype.filled;
    /** @type {?} */
    CalendarComponent.prototype.inputFieldValue;
    /** @type {?} */
    CalendarComponent.prototype._minDate;
    /** @type {?} */
    CalendarComponent.prototype._maxDate;
    /** @type {?} */
    CalendarComponent.prototype._isValid;
    /** @type {?} */
    CalendarComponent.prototype.onModelChange;
    /** @type {?} */
    CalendarComponent.prototype.onModelTouched;
    /** @type {?} */
    CalendarComponent.prototype.el;
    /** @type {?} */
    CalendarComponent.prototype.domHandler;
    /** @type {?} */
    CalendarComponent.prototype.renderer;
    /** @type {?} */
    CalendarComponent.prototype.cd;
}
