import { DomSanitizer } from '@angular/platform-browser';
import { FileUploader } from './file-upload/file-uploader.class';
import { Component, Input, Output, EventEmitter, ViewChild, ViewChildren, Inject, Optional } from '@angular/core';
import { FileType } from './file-upload/file-type.class';
import { FileSize } from './file-upload/file-size.class';
import { FileSelectDirective } from './file-upload/file-select.directive';
import { CUI_ROOT_CONFIG } from '../../consoleui-config';
/**
 * @param {?} name
 * @return {?}
 */
export function getCookie(name) {
    const /** @type {?} */ cookies = document.cookie;
    const /** @type {?} */ list = cookies.split("; "); // 解析出名/值对列表
    for (let /** @type {?} */ i = 0; i < list.length; i++) {
        const /** @type {?} */ arr = list[i].split("="); // 解析出名和值
        if (arr[0] == name) {
            return decodeURIComponent(arr[1]); // 对cookie值解码
        }
    }
    return "";
}
export class FileuploadComponent {
    /**
     * @param {?} sanitizer
     * @param {?} cuiRootConfig
     */
    constructor(sanitizer, cuiRootConfig) {
        this.sanitizer = sanitizer;
        this.cuiRootConfig = cuiRootConfig;
        this.method = 'POST';
        this.withCredentials = true;
        // @Input()
        this._mode = 'advanced';
        this.maxFileSize = 1024 * 1024;
        this.showErr = true;
        this.isCsrfTokenHeader = true;
        this.uploadComplete = new EventEmitter();
        this.error = new EventEmitter();
        this._modeMapAccepts = {
            'doc': ".doc, .docx, .ppt, .pptx, .xls, .xlsx, .pdf, .txt",
            'video': "video/\*",
            'image': "image/\*",
            'zip': ".zip",
        };
        this.basePath = '';
        if (cuiRootConfig && cuiRootConfig.fileUpload && cuiRootConfig.fileUpload.basePath) {
            const basePath = cuiRootConfig.fileUpload.basePath;
            this.basePath = basePath.endsWith("/") ? basePath.substring(0, basePath.length - 1) : basePath;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set mode(value) {
        this._mode = value;
        if (!this.accept && value) {
            this.accept = this._modeMapAccepts[value];
        }
    }
    /**
     * @return {?}
     */
    get mode() {
        return this._mode || 'advanced';
    }
    /**
     * @return {?}
     */
    get allowedFileType() {
        if (!this.accept) {
            return undefined;
        }
        return this.accept.split(/,|，/).map(it => FileType.getMimeClassByType(it));
    }
    /**
     * @return {?}
     */
    _getXsrfToken() {
        const /** @type {?} */ tokenName = "XSRF-TOKEN";
        return getCookie(tokenName);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        const /** @type {?} */ csrfToken = this._getXsrfToken();
        const /** @type {?} */ csrfTokenHeader = (this.isCsrfTokenHeader && !!csrfToken) ? { name: "X-XSRF-TOKEN", value: csrfToken } : undefined;
        const /** @type {?} */ headers = csrfTokenHeader ? [csrfTokenHeader] : [];
        this.uploader = new FileUploader({
            url: (this.basePath || '') + this.url,
            itemAlias: this.name,
            autoUpload: this.auto,
            // allowedFileType: this.allowedFileType,
            accept: this.accept,
            maxFileSize: this.maxFileSize,
            headers: headers
        });
        // this.uploader.
        this.uploader.onSuccessItem = (item, response, status, headers) => {
            // console.log(JSON.parse(response));
        };
        this.uploader.onErrorItem = (item, response, status, headers) => {
            // console.log({ item, response, status, headers })
            return { item, response, status, headers };
        };
        this.uploader.onCompleteAll = () => {
            this.refreshResult();
        };
        this.uploader.onWhenAddingFileFailed = (item, filter, options) => {
            let /** @type {?} */ err;
            switch (filter.name) {
                case 'fileType': {
                    err = { code: filter.name, message: `文件'${item.name}'格式不支持, 允许的格式 ${this.accept}` };
                    break;
                }
                case 'accept': {
                    err = { code: filter.name, message: `文件'${item.name}'格式不支持, 允许的格式 ${this.accept}` };
                    break;
                }
                case 'fileSize': {
                    err = { code: filter.name, message: `文件'${item.name}'大小超出限制, 最大支持 ${FileSize.prettySize(options.maxFileSize, 0)}` };
                    break;
                }
            }
            if (err) {
                this._addingErrors = this._addingErrors ? [...this._addingErrors, err] : [err];
            }
            return { item, filter, options };
        };
        // this.uploader.onAfterAddingAll = (fileItems: any): any => {
        //     if (this._addingErrors) {
        //         this.errs = this._addingErrors;
        //         this.error.emit(this.errs);
        //         this._addingErrors = [];
        //     }
        //     return { fileItems };
        // };
        if (!this.multiple) {
            this.uploader.onAfterAddingFile = (fileItem) => {
                let /** @type {?} */ ofs = this.uploader.queue.filter(it => it != fileItem);
                ofs.forEach(it => {
                    this.uploader.removeFromQueue(it);
                });
            };
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes['accept'] && this.uploader) {
            // this.uploader.options.allowedFileType = this.allowedFileType;
            this.uploader.options.accept = this.accept;
        }
        if (changes['maxFileSize'] && this.uploader) {
            this.uploader.options.maxFileSize = this.maxFileSize;
        }
    }
    /**
     * @param {?} files
     * @return {?}
     */
    onSelected(files) {
        if (this._addingErrors) {
            this.errs = this._addingErrors;
            this.error.emit(this.errs);
            this._addingErrors = [];
        }
    }
    /**
     * @return {?}
     */
    hasFiles() {
        return this.uploader && this.uploader.queue.length > 0;
    }
    /**
     * @return {?}
     */
    hasNotUploadFiles() {
        return this.uploader && this.uploader.getNotUploadedItems().length > 0;
    }
    /**
     * @param {?} file
     * @return {?}
     */
    isImage(file) {
        return /^image\//.test(file.type);
    }
    /**
     * @param {?} file
     * @return {?}
     */
    getSafeUrl(file) {
        return this.sanitizer.bypassSecurityTrustUrl(window.URL.createObjectURL(file));
    }
    /**
     * @param {?} item
     * @return {?}
     */
    removeFromQueue(item) {
        this.uploader.removeFromQueue(item);
        this.refreshResult();
    }
    /**
     * @param {?=} clearFile
     * @return {?}
     */
    clearQueue(clearFile) {
        if (clearFile && this.fileSelectors) {
            this.fileSelectors.forEach((it) => {
                it.clear();
            });
        }
        this.uploader.clearQueue();
        this.refreshResult();
    }
    /**
     * @return {?}
     */
    refreshResult() {
        this.results = this.uploader.queue.filter(item => item.isSuccess)
            .map(item => {
            if (item._xhr && item._xhr.response) {
                try {
                    return JSON.parse(item._xhr.response);
                }
                catch (e) {
                    return null;
                }
            }
            else {
                return null;
            }
        }).filter(item => item !== null);
        this.uploadComplete.emit(this.multiple ? this.results : (this.results && this.results.length > 0 ? this.results[0] : undefined));
    }
    /**
     * @return {?}
     */
    chooseFile() {
        if (this.file) {
            ((this.file.nativeElement)).click();
        }
    }
}
FileuploadComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-fileupload',
                templateUrl: './fileupload.component.html',
                styleUrls: ['./fileupload.component.scss']
            },] },
];
/**
 * @nocollapse
 */
FileuploadComponent.ctorParameters = () => [
    { type: DomSanitizer, },
    { type: undefined, decorators: [{ type: Inject, args: [CUI_ROOT_CONFIG,] }, { type: Optional },] },
];
FileuploadComponent.propDecorators = {
    'name': [{ type: Input },],
    'url': [{ type: Input },],
    'method': [{ type: Input },],
    'withCredentials': [{ type: Input },],
    'multiple': [{ type: Input },],
    'auto': [{ type: Input },],
    'accept': [{ type: Input },],
    'maxFileSize': [{ type: Input },],
    'imageHolder': [{ type: Input },],
    'showErr': [{ type: Input },],
    'isCsrfTokenHeader': [{ type: Input },],
    'uploadComplete': [{ type: Output },],
    'error': [{ type: Output },],
    'fileSelectors': [{ type: ViewChildren, args: [FileSelectDirective,] },],
    'file': [{ type: ViewChild, args: ['file',] },],
    'mode': [{ type: Input },],
};
function FileuploadComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    FileuploadComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    FileuploadComponent.ctorParameters;
    /** @type {?} */
    FileuploadComponent.propDecorators;
    /** @type {?} */
    FileuploadComponent.prototype.name;
    /** @type {?} */
    FileuploadComponent.prototype.url;
    /** @type {?} */
    FileuploadComponent.prototype.method;
    /** @type {?} */
    FileuploadComponent.prototype.withCredentials;
    /** @type {?} */
    FileuploadComponent.prototype.multiple;
    /** @type {?} */
    FileuploadComponent.prototype.auto;
    /** @type {?} */
    FileuploadComponent.prototype.accept;
    /** @type {?} */
    FileuploadComponent.prototype._mode;
    /** @type {?} */
    FileuploadComponent.prototype.maxFileSize;
    /** @type {?} */
    FileuploadComponent.prototype.imageHolder;
    /** @type {?} */
    FileuploadComponent.prototype.showErr;
    /** @type {?} */
    FileuploadComponent.prototype.isCsrfTokenHeader;
    /** @type {?} */
    FileuploadComponent.prototype.uploadComplete;
    /** @type {?} */
    FileuploadComponent.prototype.error;
    /** @type {?} */
    FileuploadComponent.prototype.fileSelectors;
    /** @type {?} */
    FileuploadComponent.prototype.file;
    /** @type {?} */
    FileuploadComponent.prototype.uploader;
    /** @type {?} */
    FileuploadComponent.prototype.results;
    /** @type {?} */
    FileuploadComponent.prototype._addingErrors;
    /** @type {?} */
    FileuploadComponent.prototype.errs;
    /** @type {?} */
    FileuploadComponent.prototype._modeMapAccepts;
    /** @type {?} */
    FileuploadComponent.prototype.basePath;
    /** @type {?} */
    FileuploadComponent.prototype.sanitizer;
    /** @type {?} */
    FileuploadComponent.prototype.cuiRootConfig;
}
