import { Component, Input, Output, EventEmitter, forwardRef, ViewChild, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export const /** @type {?} */ RADIO_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => RadioComponent),
    multi: true
};
export class RadioComponent {
    /**
     * @param {?} cd
     */
    constructor(cd) {
        this.cd = cd;
        this.inline = false;
        this.onClick = new EventEmitter();
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.input = (this.inputViewChild.nativeElement);
    }
    /**
     * @return {?}
     */
    handleClick() {
        if (!this.disabled) {
            this.onClick.emit(null);
            this.select();
        }
    }
    /**
     * @return {?}
     */
    select() {
        if (!this.disabled) {
            this.input.checked = true;
            this.checked = true;
            this.onModelChange(this.value || this.checked);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.checked = this.value ? (value == this.value) : !!value;
        if (this.input) {
            this.input.checked = this.checked;
        }
        this.cd.markForCheck();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setDisabledState(val) {
        this.disabled = val;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        this.focused = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBlur(event) {
        this.focused = false;
        this.onModelTouched();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onChange(event) {
        this.select();
    }
}
RadioComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-radio, [cui-radio]',
                templateUrl: './radio.component.html',
                styleUrls: ['./radio.component.scss'],
                providers: [RADIO_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
RadioComponent.ctorParameters = () => [
    { type: ChangeDetectorRef, },
];
RadioComponent.propDecorators = {
    'value': [{ type: Input },],
    'name': [{ type: Input },],
    'disabled': [{ type: Input },],
    'label': [{ type: Input },],
    'tabindex': [{ type: Input },],
    'inputId': [{ type: Input },],
    'style': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'inline': [{ type: Input },],
    'onClick': [{ type: Output },],
    'inputViewChild': [{ type: ViewChild, args: ['rb',] },],
};
function RadioComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    RadioComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    RadioComponent.ctorParameters;
    /** @type {?} */
    RadioComponent.propDecorators;
    /** @type {?} */
    RadioComponent.prototype.value;
    /** @type {?} */
    RadioComponent.prototype.name;
    /** @type {?} */
    RadioComponent.prototype.disabled;
    /** @type {?} */
    RadioComponent.prototype.label;
    /** @type {?} */
    RadioComponent.prototype.tabindex;
    /** @type {?} */
    RadioComponent.prototype.inputId;
    /** @type {?} */
    RadioComponent.prototype.style;
    /** @type {?} */
    RadioComponent.prototype.styleClass;
    /** @type {?} */
    RadioComponent.prototype.inline;
    /** @type {?} */
    RadioComponent.prototype.onClick;
    /** @type {?} */
    RadioComponent.prototype.inputViewChild;
    /** @type {?} */
    RadioComponent.prototype.input;
    /** @type {?} */
    RadioComponent.prototype.checked;
    /** @type {?} */
    RadioComponent.prototype.focused;
    /** @type {?} */
    RadioComponent.prototype.onModelChange;
    /** @type {?} */
    RadioComponent.prototype.onModelTouched;
    /** @type {?} */
    RadioComponent.prototype.cd;
}
