import { Component, forwardRef, Input, Output, EventEmitter, ViewChild, ElementRef, ContentChild, Renderer2 } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { DomHandler } from '../../core/dom/dom-handler';
import { ChangeDetectorRef } from '@angular/core';
import { ObjectUtils } from '../../core/utils/object-utils';
export const /** @type {?} */ SELECT_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => SelectComponent),
    multi: true
};
export class SelectComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} renderer
     * @param {?} cd
     */
    constructor(el, domHandler, renderer, cd) {
        this.el = el;
        this.domHandler = domHandler;
        this.renderer = renderer;
        this.cd = cd;
        this.scrollHeight = '200px';
        this.autoWidth = true;
        this.filterBy = 'label';
        this.onChange = new EventEmitter();
        this.onFocus = new EventEmitter();
        this.onBlur = new EventEmitter();
        this.panelVisible = false;
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        /*this.templates.forEach((item) => {
          switch (item.getType()) {
            case 'item':
              this.itemTemplate = item.template;
              break;
    
            default:
              this.itemTemplate = item.template;
              break;
          }
        });*/
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.optionsToDisplay = this.options;
        this.updateSelectedOption(null);
    }
    /**
     * @return {?}
     */
    get options() {
        return this._options;
    }
    /**
     * @param {?} opts
     * @return {?}
     */
    set options(opts) {
        this._options = opts;
        this.optionsToDisplay = this._options;
        this.updateSelectedOption(this.value);
        this.optionsChanged = true;
        if (this.filterValue && this.filterValue.length) {
            this.activateFilter();
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.container = (this.containerViewChild.nativeElement);
        this.panel = (this.panelViewChild.nativeElement);
        this.itemsWrapper = (this.itemsWrapperViewChild.nativeElement);
        this.updateDimensions();
        this.initialized = true;
        if (this.appendTo) {
            if (this.appendTo === 'body') {
                document.body.appendChild(this.panel);
            }
            else {
                this.domHandler.appendChild(this.panel, this.appendTo);
            }
        }
    }
    /**
     * @return {?}
     */
    get label() {
        return (this.selectedOption ? this.selectedOption.label : this.placeholder);
    }
    /**
     * @return {?}
     */
    updateEditableLabel() {
        if (this.editableInputViewChild && this.editableInputViewChild.nativeElement) {
            this.editableInputViewChild.nativeElement.value = this.value || (this.selectedOption ? this.selectedOption.label : '');
        }
    }
    /**
     * @param {?} event
     * @param {?} option
     * @return {?}
     */
    onItemClick(event, option) {
        this.itemClick = true;
        this.selectItem(event, option);
        this.focusViewChild.nativeElement.focus();
        this.hide();
    }
    /**
     * @param {?} event
     * @param {?} option
     * @return {?}
     */
    selectItem(event, option) {
        this.selectedOption = option;
        this.value = option.value;
        this.onModelChange(this.value);
        this.updateEditableLabel();
        this.onChange.emit({
            originalEvent: event,
            value: this.value
        });
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        if (this.shown) {
            this.onShow();
            this.shown = false;
        }
        if (this.optionsChanged && this.panelVisible) {
            this.optionsChanged = false;
            this.alignPanel();
        }
        if (this.selectedOptionUpdated && this.itemsWrapper) {
            let /** @type {?} */ selectedItem = this.domHandler.findSingle(this.panel, 'li.ui-state-highlight');
            if (selectedItem) {
                this.domHandler.scrollInView(this.itemsWrapper, this.domHandler.findSingle(this.panel, 'li.ui-state-highlight'));
            }
            this.selectedOptionUpdated = false;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (this.filter) {
            this.resetFilter();
        }
        this.value = value;
        this.updateSelectedOption(value);
        this.updateEditableLabel();
        this.cd.markForCheck();
    }
    /**
     * @return {?}
     */
    resetFilter() {
        if (this.filterViewChild && this.filterViewChild.nativeElement) {
            this.filterViewChild.nativeElement.value = '';
        }
        this.optionsToDisplay = this.options;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    updateSelectedOption(val) {
        this.selectedOption = this.findOption(val, this.optionsToDisplay);
        if (!this.placeholder && !this.selectedOption && this.optionsToDisplay && this.optionsToDisplay.length && !this.editable) {
            this.selectedOption = this.optionsToDisplay[0];
        }
        this.selectedOptionUpdated = true;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setDisabledState(val) {
        this.disabled = val;
    }
    /**
     * @return {?}
     */
    updateDimensions() {
        if (this.autoWidth) {
            let /** @type {?} */ select = this.domHandler.findSingle(this.el.nativeElement, 'select');
            console.log(select, select.offsetWidth);
            if (!this.style || (!this.style['width'] && !this.style['min-width'])) {
                this.el.nativeElement.children[0].style.width = select.offsetWidth + 30 + 'px';
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseclick(event) {
        if (this.disabled || this.readonly) {
            return;
        }
        this.selfClick = true;
        if (!this.itemClick) {
            this.focusViewChild.nativeElement.focus();
            if (this.panelVisible) {
                this.hide();
            }
            else {
                this.show();
                if (this.filterViewChild != undefined) {
                    setTimeout(() => {
                        this.filterViewChild.nativeElement.focus();
                    }, 200);
                }
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onEditableInputClick(event) {
        this.itemClick = true;
        this.bindDocumentClickListener();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onEditableInputFocus(event) {
        this.focus = true;
        this.hide();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onEditableInputChange(event) {
        this.value = event.target.value;
        this.updateSelectedOption(this.value);
        this.onModelChange(this.value);
        this.onChange.emit({
            originalEvent: event,
            value: this.value
        });
    }
    /**
     * @return {?}
     */
    onShow() {
        if (this.options && this.options.length) {
            this.alignPanel();
            this.bindDocumentClickListener();
            let /** @type {?} */ selectedListItem = this.domHandler.findSingle(this.itemsWrapper, '.ui-dropdown-item.ui-state-highlight');
            if (selectedListItem) {
                this.domHandler.scrollInView(this.itemsWrapper, selectedListItem);
            }
        }
    }
    /**
     * @return {?}
     */
    show() {
        if (this.appendTo) {
            this.panel.style.minWidth = this.domHandler.getWidth(this.container) + 'px';
        }
        this.panel.style.zIndex = String(++DomHandler.zindex);
        this.panelVisible = true;
        this.shown = true;
    }
    /**
     * @return {?}
     */
    hide() {
        this.panelVisible = false;
    }
    /**
     * @return {?}
     */
    alignPanel() {
        if (this.appendTo) {
            this.domHandler.absolutePosition(this.panel, this.container);
        }
        else {
            this.domHandler.relativePosition(this.panel, this.container);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputFocus(event) {
        this.focus = true;
        this.onFocus.emit(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputBlur(event) {
        this.focus = false;
        this.onModelTouched();
        this.onBlur.emit(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeydown(event) {
        if (this.readonly) {
            return;
        }
        let /** @type {?} */ selectedItemIndex = this.selectedOption ? this.findOptionIndex(this.selectedOption.value, this.optionsToDisplay) : -1;
        switch (event.which) {
            // down
            case 40:
                if (!this.panelVisible && event.altKey) {
                    this.show();
                }
                else {
                    if (selectedItemIndex != -1) {
                        let /** @type {?} */ nextItemIndex = selectedItemIndex + 1;
                        if (nextItemIndex != (this.optionsToDisplay.length)) {
                            this.selectedOption = this.optionsToDisplay[nextItemIndex];
                            this.selectedOptionUpdated = true;
                            this.selectItem(event, this.selectedOption);
                        }
                    }
                    else if (this.optionsToDisplay) {
                        this.selectedOption = this.optionsToDisplay[0];
                    }
                }
                event.preventDefault();
                break;
            // up
            case 38:
                if (selectedItemIndex > 0) {
                    let /** @type {?} */ prevItemIndex = selectedItemIndex - 1;
                    this.selectedOption = this.optionsToDisplay[prevItemIndex];
                    this.selectedOptionUpdated = true;
                    this.selectItem(event, this.selectedOption);
                }
                event.preventDefault();
                break;
            // space
            case 32:
                this.panelVisible = !this.panelVisible;
                event.preventDefault();
                break;
            // enter
            case 13:
                this.hide();
                event.preventDefault();
                break;
            // escape and tab
            case 27:
            case 9:
                this.panelVisible = false;
                break;
        }
    }
    /**
     * @param {?} val
     * @param {?} opts
     * @return {?}
     */
    findOptionIndex(val, opts) {
        let /** @type {?} */ index = -1;
        if (opts) {
            for (let /** @type {?} */ i = 0; i < opts.length; i++) {
                if ((val == null && opts[i].value == null) || ObjectUtils.equals(val, opts[i].value, this.dataKey)) {
                    index = i;
                    break;
                }
            }
        }
        return index;
    }
    /**
     * @param {?} val
     * @param {?} opts
     * @return {?}
     */
    findOption(val, opts) {
        let /** @type {?} */ index = this.findOptionIndex(val, opts);
        return (index != -1) ? opts[index] : null;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFilter(event) {
        let /** @type {?} */ inputValue = event.target.value.toLowerCase();
        if (inputValue && inputValue.length) {
            this.filterValue = inputValue;
            this.activateFilter();
        }
        else {
            this.filterValue = null;
            this.optionsToDisplay = this.options;
        }
        this.optionsChanged = true;
    }
    /**
     * @return {?}
     */
    activateFilter() {
        let /** @type {?} */ searchFields = this.filterBy.split(',');
        if (this.options && this.options.length) {
            this.optionsToDisplay = ObjectUtils.filter(this.options, searchFields, this.filterValue);
            this.optionsChanged = true;
        }
    }
    /**
     * @return {?}
     */
    applyFocus() {
        if (this.editable) {
            this.domHandler.findSingle(this.el.nativeElement, '.ui-dropdown-label.ui-inputtext').focus();
        }
        else {
            this.domHandler.findSingle(this.el.nativeElement, 'input[readonly]').focus();
        }
    }
    /**
     * @return {?}
     */
    bindDocumentClickListener() {
        if (!this.documentClickListener) {
            this.documentClickListener = this.renderer.listen('document', 'click', () => {
                if (!this.selfClick && !this.itemClick) {
                    this.panelVisible = false;
                    this.unbindDocumentClickListener();
                }
                this.selfClick = false;
                this.itemClick = false;
                this.cd.markForCheck();
            });
        }
    }
    /**
     * @return {?}
     */
    unbindDocumentClickListener() {
        if (this.documentClickListener) {
            this.documentClickListener();
            this.documentClickListener = null;
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.initialized = false;
        this.unbindDocumentClickListener();
        if (this.appendTo) {
            this.el.nativeElement.appendChild(this.panel);
        }
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-select',
                templateUrl: './select.component.html',
                styleUrls: ['./select.component.scss'],
                providers: [SELECT_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
SelectComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: Renderer2, },
    { type: ChangeDetectorRef, },
];
SelectComponent.propDecorators = {
    'scrollHeight': [{ type: Input },],
    'filter': [{ type: Input },],
    'style': [{ type: Input },],
    'panelStyle': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'panelStyleClass': [{ type: Input },],
    'disabled': [{ type: Input },],
    'readonly': [{ type: Input },],
    'autoWidth': [{ type: Input },],
    'required': [{ type: Input },],
    'editable': [{ type: Input },],
    'appendTo': [{ type: Input },],
    'tabindex': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'filterPlaceholder': [{ type: Input },],
    'inputId': [{ type: Input },],
    'dataKey': [{ type: Input },],
    'filterBy': [{ type: Input },],
    'onChange': [{ type: Output },],
    'onFocus': [{ type: Output },],
    'onBlur': [{ type: Output },],
    'containerViewChild': [{ type: ViewChild, args: ['container',] },],
    'panelViewChild': [{ type: ViewChild, args: ['panel',] },],
    'itemsWrapperViewChild': [{ type: ViewChild, args: ['itemswrapper',] },],
    'filterViewChild': [{ type: ViewChild, args: ['filter',] },],
    'focusViewChild': [{ type: ViewChild, args: ['in',] },],
    'editableInputViewChild': [{ type: ViewChild, args: ['editableInput',] },],
    'itemTemplate': [{ type: ContentChild, args: ['itemTemplate',] },],
    'options': [{ type: Input },],
};
function SelectComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    SelectComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SelectComponent.ctorParameters;
    /** @type {?} */
    SelectComponent.propDecorators;
    /** @type {?} */
    SelectComponent.prototype.scrollHeight;
    /** @type {?} */
    SelectComponent.prototype.filter;
    /** @type {?} */
    SelectComponent.prototype.style;
    /** @type {?} */
    SelectComponent.prototype.panelStyle;
    /** @type {?} */
    SelectComponent.prototype.styleClass;
    /** @type {?} */
    SelectComponent.prototype.panelStyleClass;
    /** @type {?} */
    SelectComponent.prototype.disabled;
    /** @type {?} */
    SelectComponent.prototype.readonly;
    /** @type {?} */
    SelectComponent.prototype.autoWidth;
    /** @type {?} */
    SelectComponent.prototype.required;
    /** @type {?} */
    SelectComponent.prototype.editable;
    /** @type {?} */
    SelectComponent.prototype.appendTo;
    /** @type {?} */
    SelectComponent.prototype.tabindex;
    /** @type {?} */
    SelectComponent.prototype.placeholder;
    /** @type {?} */
    SelectComponent.prototype.filterPlaceholder;
    /** @type {?} */
    SelectComponent.prototype.inputId;
    /** @type {?} */
    SelectComponent.prototype.dataKey;
    /** @type {?} */
    SelectComponent.prototype.filterBy;
    /** @type {?} */
    SelectComponent.prototype.onChange;
    /** @type {?} */
    SelectComponent.prototype.onFocus;
    /** @type {?} */
    SelectComponent.prototype.onBlur;
    /** @type {?} */
    SelectComponent.prototype.containerViewChild;
    /** @type {?} */
    SelectComponent.prototype.panelViewChild;
    /** @type {?} */
    SelectComponent.prototype.itemsWrapperViewChild;
    /** @type {?} */
    SelectComponent.prototype.filterViewChild;
    /** @type {?} */
    SelectComponent.prototype.focusViewChild;
    /** @type {?} */
    SelectComponent.prototype.editableInputViewChild;
    /** @type {?} */
    SelectComponent.prototype.itemTemplate;
    /** @type {?} */
    SelectComponent.prototype.selectedOption;
    /** @type {?} */
    SelectComponent.prototype._options;
    /** @type {?} */
    SelectComponent.prototype.value;
    /** @type {?} */
    SelectComponent.prototype.optionsToDisplay;
    /** @type {?} */
    SelectComponent.prototype.hover;
    /** @type {?} */
    SelectComponent.prototype.focus;
    /** @type {?} */
    SelectComponent.prototype.panelVisible;
    /** @type {?} */
    SelectComponent.prototype.shown;
    /** @type {?} */
    SelectComponent.prototype.documentClickListener;
    /** @type {?} */
    SelectComponent.prototype.optionsChanged;
    /** @type {?} */
    SelectComponent.prototype.panel;
    /** @type {?} */
    SelectComponent.prototype.container;
    /** @type {?} */
    SelectComponent.prototype.itemsWrapper;
    /** @type {?} */
    SelectComponent.prototype.initialized;
    /** @type {?} */
    SelectComponent.prototype.selfClick;
    /** @type {?} */
    SelectComponent.prototype.itemClick;
    /** @type {?} */
    SelectComponent.prototype.hoveredItem;
    /** @type {?} */
    SelectComponent.prototype.selectedOptionUpdated;
    /** @type {?} */
    SelectComponent.prototype.filterValue;
    /** @type {?} */
    SelectComponent.prototype.onModelChange;
    /** @type {?} */
    SelectComponent.prototype.onModelTouched;
    /** @type {?} */
    SelectComponent.prototype.el;
    /** @type {?} */
    SelectComponent.prototype.domHandler;
    /** @type {?} */
    SelectComponent.prototype.renderer;
    /** @type {?} */
    SelectComponent.prototype.cd;
}
