import { DomHandler } from './../../core/dom/dom-handler';
import { Component, Input, Output, EventEmitter, forwardRef, Renderer2, ChangeDetectorRef, ElementRef, ViewChild } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export const /** @type {?} */ SELECT_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => TreeSelectComponent),
    multi: true
};
export class TreeSelectComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} renderer
     * @param {?} cd
     */
    constructor(el, domHandler, renderer, cd) {
        this.el = el;
        this.domHandler = domHandler;
        this.renderer = renderer;
        this.cd = cd;
        this.scrollHeight = '200px';
        this.autoWidth = true;
        this.filterBy = 'label';
        this.onChange = new EventEmitter();
        this.onFocus = new EventEmitter();
        this.onBlur = new EventEmitter();
        this.panelVisible = false;
        this.isClearClick = false;
        this.open = false;
        this.change = new EventEmitter();
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
        this.defaultOptions = {};
    }
    /**
     * @return {?}
     */
    get label() {
        // return (this.selected ? this.selected.label : this.placeholder);
        return this.value ? this.value[this.config.data.key.label] : this.placeholder;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.config = Object.assign({}, this.defaultOptions, this.config);
        // this.optionsToDisplay = this.options;
        // this.updateSelectedOption(null);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.container = (this.containerViewChild.nativeElement);
        this.panel = (this.panelViewChild.nativeElement);
        this.itemsWrapper = (this.itemsWrapperViewChild.nativeElement);
        this.updateDimensions();
        this.initialized = true;
        if (this.appendTo) {
            if (this.appendTo === 'body') {
                document.body.appendChild(this.panel);
            }
            else {
                this.domHandler.appendChild(this.panel, this.appendTo);
            }
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        if (this.shown) {
            this.onShow();
            this.shown = false;
        }
        /* if (this.optionsChanged && this.panelVisible) {
          this.optionsChanged = false;
          this.alignPanel();
        } */
        if (this.selectedOptionUpdated && this.itemsWrapper) {
            let /** @type {?} */ selectedItem = this.domHandler.findSingle(this.panel, 'li.ui-state-highlight');
            if (selectedItem) {
                this.domHandler.scrollInView(this.itemsWrapper, this.domHandler.findSingle(this.panel, 'li.ui-state-highlight'));
            }
            this.selectedOptionUpdated = false;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (this.filter) {
            this.resetFilter();
        }
        this.value = value;
        // this.updateSelectedOption(value);
        // this.updateEditableLabel();
        this.cd.markForCheck();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    /**
     * @return {?}
     */
    resetFilter() {
        if (this.filterViewChild && this.filterViewChild.nativeElement) {
            this.filterViewChild.nativeElement.value = '';
        }
        // this.optionsToDisplay = this.options;
    }
    /**
     * @return {?}
     */
    updateSelected() {
    }
    /**
     * @param {?} nodes
     * @return {?}
     */
    onSelectionChange(nodes) {
        console.log('Tree-Select Component slected nodes: ', nodes);
        if (!nodes) {
            return;
        }
        this.value = nodes[0].data;
        this.onModelChange(this.value);
        this.hide();
        this.change.emit(this.value);
    }
    /**
     * @return {?}
     */
    updateDimensions() {
        /* if (this.autoWidth) {
          let select = this.domHandler.findSingle(this.el.nativeElement, 'select');
          console.log(select, select.offsetWidth)
          if (!this.style || (!this.style['width'] && !this.style['min-width'])) {
            this.el.nativeElement.children[0].style.width = select.offsetWidth + 30 + 'px';
          }
        } */
    }
    /**
     * @return {?}
     */
    onItemClick() {
        this.itemClick = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseClick(event) {
        if (this.disabled || this.readonly) {
            return;
        }
        this.selfClick = true;
        if (this.isClearClick) {
            this.isClearClick = false;
            return;
        }
        if (!this.itemClick) {
            this.focusViewChild.nativeElement.focus();
            if (this.panelVisible) {
                this.hide();
            }
            else {
                this.show();
                if (this.filterViewChild != undefined) {
                    setTimeout(() => {
                        this.filterViewChild.nativeElement.focus();
                    }, 200);
                }
            }
        }
    }
    /**
     * @return {?}
     */
    onShow() {
        // TODO: if tree inited
        this.alignPanel();
        this.bindDocumentClickListener();
        // scroll to selected
    }
    /**
     * @return {?}
     */
    show() {
        if (this.appendTo) {
            this.panel.style.minWidth = this.domHandler.getWidth(this.container) + 'px';
        }
        this.panel.style.zIndex = String(++DomHandler.zindex);
        this.panelVisible = true;
        this.shown = true;
    }
    /**
     * @return {?}
     */
    hide() {
        this.panelVisible = false;
    }
    /**
     * @return {?}
     */
    alignPanel() {
        if (this.appendTo) {
            this.domHandler.absolutePosition(this.panel, this.container);
        }
        else {
            this.domHandler.relativePosition(this.panel, this.container);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputFocus(event) {
        this.focus = true;
        this.onFocus.emit(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInputBlur(event) {
        this.focus = false;
        this.onModelTouched();
        this.onBlur.emit(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeydown(event) {
        if (this.readonly) {
            return;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFilter(event) {
    }
    /**
     * @return {?}
     */
    applyFocus() {
        this.domHandler.findSingle(this.el.nativeElement, 'input[readonly]').focus();
    }
    /**
     * @return {?}
     */
    bindDocumentClickListener() {
        if (!this.documentClickListener) {
            this.documentClickListener = this.renderer.listen('document', 'click', () => {
                if (!this.selfClick && !this.itemClick) {
                    this.panelVisible = false;
                    this.unbindDocumentClickListener();
                }
                this.selfClick = false;
                this.itemClick = false;
                this.cd.markForCheck();
            });
        }
    }
    /**
     * @return {?}
     */
    unbindDocumentClickListener() {
        if (this.documentClickListener) {
            this.documentClickListener();
            this.documentClickListener = null;
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.initialized = false;
        this.unbindDocumentClickListener();
        if (this.appendTo) {
            this.el.nativeElement.appendChild(this.panel);
        }
    }
    /**
     * @return {?}
     */
    clearValue() {
        this.isClearClick = true;
        if (this.tree) {
            this.tree.selection = [];
            this.value = undefined;
            this.onModelChange(this.value);
            this.hide();
            this.change.emit(this.value);
        }
    }
}
TreeSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tree-select',
                templateUrl: './tree-select.component.html',
                styleUrls: ['./tree-select.component.scss'],
                providers: [SELECT_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
TreeSelectComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: Renderer2, },
    { type: ChangeDetectorRef, },
];
TreeSelectComponent.propDecorators = {
    'scrollHeight': [{ type: Input },],
    'filter': [{ type: Input },],
    'style': [{ type: Input },],
    'panelStyle': [{ type: Input },],
    'styleClass': [{ type: Input },],
    'panelStyleClass': [{ type: Input },],
    'disabled': [{ type: Input },],
    'readonly': [{ type: Input },],
    'autoWidth': [{ type: Input },],
    'required': [{ type: Input },],
    'appendTo': [{ type: Input },],
    'tabindex': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'filterPlaceholder': [{ type: Input },],
    'inputId': [{ type: Input },],
    'dataKey': [{ type: Input },],
    'filterBy': [{ type: Input },],
    'allowClear': [{ type: Input },],
    'onChange': [{ type: Output },],
    'onFocus': [{ type: Output },],
    'onBlur': [{ type: Output },],
    'containerViewChild': [{ type: ViewChild, args: ['container',] },],
    'panelViewChild': [{ type: ViewChild, args: ['panel',] },],
    'itemsWrapperViewChild': [{ type: ViewChild, args: ['itemswrapper',] },],
    'filterViewChild': [{ type: ViewChild, args: ['filter',] },],
    'focusViewChild': [{ type: ViewChild, args: ['in',] },],
    'tree': [{ type: ViewChild, args: ['tree',] },],
    'name': [{ type: Input },],
    'selected': [{ type: Input },],
    'config': [{ type: Input },],
    'change': [{ type: Output },],
};
function TreeSelectComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    TreeSelectComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    TreeSelectComponent.ctorParameters;
    /** @type {?} */
    TreeSelectComponent.propDecorators;
    /** @type {?} */
    TreeSelectComponent.prototype.scrollHeight;
    /** @type {?} */
    TreeSelectComponent.prototype.filter;
    /** @type {?} */
    TreeSelectComponent.prototype.style;
    /** @type {?} */
    TreeSelectComponent.prototype.panelStyle;
    /** @type {?} */
    TreeSelectComponent.prototype.styleClass;
    /** @type {?} */
    TreeSelectComponent.prototype.panelStyleClass;
    /** @type {?} */
    TreeSelectComponent.prototype.disabled;
    /** @type {?} */
    TreeSelectComponent.prototype.readonly;
    /** @type {?} */
    TreeSelectComponent.prototype.autoWidth;
    /** @type {?} */
    TreeSelectComponent.prototype.required;
    /** @type {?} */
    TreeSelectComponent.prototype.appendTo;
    /** @type {?} */
    TreeSelectComponent.prototype.tabindex;
    /** @type {?} */
    TreeSelectComponent.prototype.placeholder;
    /** @type {?} */
    TreeSelectComponent.prototype.filterPlaceholder;
    /** @type {?} */
    TreeSelectComponent.prototype.inputId;
    /** @type {?} */
    TreeSelectComponent.prototype.dataKey;
    /** @type {?} */
    TreeSelectComponent.prototype.filterBy;
    /** @type {?} */
    TreeSelectComponent.prototype.allowClear;
    /** @type {?} */
    TreeSelectComponent.prototype.onChange;
    /** @type {?} */
    TreeSelectComponent.prototype.onFocus;
    /** @type {?} */
    TreeSelectComponent.prototype.onBlur;
    /** @type {?} */
    TreeSelectComponent.prototype.containerViewChild;
    /** @type {?} */
    TreeSelectComponent.prototype.panelViewChild;
    /** @type {?} */
    TreeSelectComponent.prototype.itemsWrapperViewChild;
    /** @type {?} */
    TreeSelectComponent.prototype.filterViewChild;
    /** @type {?} */
    TreeSelectComponent.prototype.focusViewChild;
    /** @type {?} */
    TreeSelectComponent.prototype.tree;
    /** @type {?} */
    TreeSelectComponent.prototype.value;
    /** @type {?} */
    TreeSelectComponent.prototype.hover;
    /** @type {?} */
    TreeSelectComponent.prototype.focus;
    /** @type {?} */
    TreeSelectComponent.prototype.panelVisible;
    /** @type {?} */
    TreeSelectComponent.prototype.shown;
    /** @type {?} */
    TreeSelectComponent.prototype.documentClickListener;
    /** @type {?} */
    TreeSelectComponent.prototype.panel;
    /** @type {?} */
    TreeSelectComponent.prototype.container;
    /** @type {?} */
    TreeSelectComponent.prototype.itemsWrapper;
    /** @type {?} */
    TreeSelectComponent.prototype.initialized;
    /** @type {?} */
    TreeSelectComponent.prototype.selfClick;
    /** @type {?} */
    TreeSelectComponent.prototype.itemClick;
    /** @type {?} */
    TreeSelectComponent.prototype.hoveredItem;
    /** @type {?} */
    TreeSelectComponent.prototype.selectedOptionUpdated;
    /** @type {?} */
    TreeSelectComponent.prototype.filterValue;
    /** @type {?} */
    TreeSelectComponent.prototype.isClearClick;
    /** @type {?} */
    TreeSelectComponent.prototype.defaultOptions;
    /** @type {?} */
    TreeSelectComponent.prototype.open;
    /** @type {?} */
    TreeSelectComponent.prototype.name;
    /** @type {?} */
    TreeSelectComponent.prototype.selected;
    /** @type {?} */
    TreeSelectComponent.prototype.config;
    /** @type {?} */
    TreeSelectComponent.prototype.change;
    /** @type {?} */
    TreeSelectComponent.prototype.onModelChange;
    /** @type {?} */
    TreeSelectComponent.prototype.onModelTouched;
    /** @type {?} */
    TreeSelectComponent.prototype.el;
    /** @type {?} */
    TreeSelectComponent.prototype.domHandler;
    /** @type {?} */
    TreeSelectComponent.prototype.renderer;
    /** @type {?} */
    TreeSelectComponent.prototype.cd;
}
