import { Component, Input, Output, EventEmitter } from '@angular/core';
export class AlertComponent {
    constructor() {
        this.closable = true;
        this.valueChange = new EventEmitter();
        this.severityIconMap = {
            'primary': null,
            'secondary': null,
            'success': 'fa-check',
            'danger': 'fa-close',
            'warning': 'fa-warning',
            'info': 'fa-info-circle',
            'light': null,
            'dark': null,
        };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    hasMessages() {
        return this.value && this.value.length > 0;
    }
    /**
     * @return {?}
     */
    getSeverityClass() {
        if (this.hasMessages()) {
            return this.value[0].severity;
        }
    }
    /**
     * @return {?}
     */
    getMessageCount() {
        if (this.hasMessages) {
            return this.value.length;
        }
        else {
            return 0;
        }
    }
    /**
     * @return {?}
     */
    getIconSize() {
        return this.getMessageCount() > 2 ? 'fa-2x' : 'fa-lg';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    clear(event) {
        this.value = [];
        this.valueChange.emit(this.value);
        event.preventDefault();
    }
    /**
     * @return {?}
     */
    get icon() {
        let /** @type {?} */ icon = null;
        if (this.hasMessages()) {
            let /** @type {?} */ msg = this.value[0];
            icon = this.severityIconMap[msg.severity] || 'fa-info-circle';
        }
        return icon;
    }
}
AlertComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-alert',
                templateUrl: './alert.component.html',
                styleUrls: ['./alert.component.scss']
            },] },
];
/**
 * @nocollapse
 */
AlertComponent.ctorParameters = () => [];
AlertComponent.propDecorators = {
    'value': [{ type: Input },],
    'closable': [{ type: Input },],
    'valueChange': [{ type: Output },],
};
function AlertComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    AlertComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    AlertComponent.ctorParameters;
    /** @type {?} */
    AlertComponent.propDecorators;
    /** @type {?} */
    AlertComponent.prototype.value;
    /** @type {?} */
    AlertComponent.prototype.closable;
    /** @type {?} */
    AlertComponent.prototype.valueChange;
    /** @type {?} */
    AlertComponent.prototype.severityIconMap;
}
